
import matplotlib.pyplot as plt
from scipy.interpolate import UnivariateSpline
import numpy as np

#check modify

#rad,me=np.genfromtxt("Blackout.txt",unpack=True)
rad,me=np.genfromtxt("Blueout.txt",unpack=True)
#rad,me=np.genfromtxt("Redout.txt",unpack=True)
#rad,me=np.genfromtxt("Greenout.txt",unpack=True)
#rad,me=np.genfromtxt("Magentaout.txt",unpack=True)

lrad= np.log10(rad)
lme= np.log10(me)
 
yspl = UnivariateSpline(lrad,lme,s=0,k=4)

plt.plot(lrad,lme,'ro',label = 'data')
plt.xlabel("rad")
plt.ylabel("me")
xran = np.linspace(lrad[0],lrad[-1],1000)


plt.plot(xran, yspl(xran))
plt.xlabel("rad")
plt.ylabel("slope")

plt.grid(True)
plt.minorticks_on()
plt.savefig("Data.png")

plt.pause(1.5)

plt.clf()

yspl2d = yspl.derivative(n=2)
yspl1d = yspl.derivative(n=1)

plt.plot(xran,yspl2d(xran))
plt.xlabel("rad")
plt.ylabel("second derivative")


plt.grid(True)
plt.minorticks_on()
plt.savefig("BreakSpl.png")



plt.pause(1.5)

plt.clf()



dev1 = yspl1d(xran)
dev2 = yspl2d(xran)

kappa = np.abs(dev2)/(1 + dev1**2)**(3/2)

plt.plot(xran,kappa)
plt.xlabel("Radius")
plt.ylabel("Kappa")


plt.grid(True)
plt.minorticks_on()
plt.savefig("KappaSpl.png")



idx = np.where(yspl2d(xran) == min(yspl2d(xran)))

brad = xran[idx[0][0]]

rad = 10**(brad)

print("break radius = ",rad)










import pytest
import os


import subprocess as sp



from galfitools.sky.GalfitSky import galfitSky

from galfitools.sky.Sky import sky

from galfitools.sky.SkyDs9 import SkyDs9
from galfitools.sky.SkyRing import SkyRing

def test_galfitSky():

    imgname =   "A1656-1-3.sbProf.fits"
    maskfile = "mask.sbProf.fits" 
    path="tests/"
    imgname = path+imgname
    maskfile = path+maskfile
    mgzpt =  21.817
    scale = 0.68 

    X = 169 
    Y = 544 

    initsky = 376 


    galfitSky(imgname, maskfile, mgzpt, scale, X, Y, initsky)



    assert os.path.isfile("sky.txt")

    if os.path.isfile("sky.txt"):
        os.remove("sky.txt")





    return None


def test_sky():



    imgname =   "A1656-1-3.sbProf.fits"
    maskfile = "mask.sbProf.fits" 
    filereg = "ds9.sky.reg"
    path="tests/"
    imgname = path+imgname
    maskfile = path+maskfile
    filereg = path+filereg
 
    mean, sig = sky(imgname, maskfile, filereg)

    tol = 1e-2

    result1 =  369.717 
    result2 = 4.407 



    diffsky1 = abs(mean -result1)

    assert diffsky1 < tol

    diffsky2 = abs(sig -result2)

    assert diffsky2 < tol



    return None



def test_SkyDs9():

    ImageFile = "A671.gtMakeMask.maskds9.masksky.fits" 
    RegFile = "skyDs9.reg" 

    path="tests/"
    ImageFile = path+ImageFile
    RegFile = path+RegFile
    maskfile = "none"
 

    mean, sig = SkyDs9(ImageFile, RegFile, maskfile, outliers=True) 


    tol = 1e-2

    result1 = 1146.627
    result2 = 6.919



    diffsky1 = abs(mean -result1)

    assert diffsky1 < tol

    diffsky2 = abs(sig -result2)

    assert diffsky2 < tol



    return None




def test_skyring():

    image = "A1656-1-3.sbProf.fits" 
    ds9regfile = "skyring.reg" 
    mask = "mask.sbProf.fits" 

    path="tests/"
    image = path+image 
    ds9regfile= path+ds9regfile
    mask=path+mask

    width = 20 
    center = False



    ##end input
    mean, std, median, rad = SkyRing(image, mask, ds9regfile, width, center,outliers=True)

    tol = 1e-2


    #update numbers
    result1 = 370.41 
    result2 = 4.43



    diffsky1 = abs(mean -result1)

    assert diffsky1 < tol

    diffsky2 = abs(std -result2)

    assert diffsky2 < tol


    #assert os.path.isfile("ring.fits")

    if os.path.isfile("skyring.fits"):
        os.remove("skyring.fits")

    #assert os.path.isfile("ringmask.fits")

    #if os.path.isfile("ringmask.fits"):
    #    os.remove("ringmask.fits")




    return None











#! /usr/bin/env python3

import numpy as np
import matplotlib.pyplot as plt
import sys
from astropy.io import fits

import argparse

from galfitools.galin.MaskDs9 import GetAxis 

def makeSim(image, GAIN, skymean, skystd, newimage)-> None: 


    sizex, sizey = GetAxis(image) 

    hdu = fits.open(image)

    img = hdu[0].data

    eimg = GAIN*img

    noisyimg = np.random.poisson(eimg)

    pimg = noisyimg/GAIN

    hdu[0].data = pimg  

    hdu.writeto("poissonimg.fits", overwrite=True)

    sky = np.random.normal(skymean, skystd, (sizex,sizey))
   

    hdusky = fits.PrimaryHDU()
    hdusky.data = sky

    hdusky.writeto("skynoise.fits", overwrite = True)



    #sumar las dos partes:

    imgsim = pimg + sky


    hdu[0].data = imgsim

    hdu.writeto(newimage, overwrite = True)


    hdu.close()



#end of program
if __name__ == '__main__':
    main()









import pytest
import os


import subprocess as sp



from galfitools.mge.mge2galfit import mge2gal
from galfitools.mge.SbProf import sbProf



def test_mge2gal():


    ##################################
    path="tests/"
    galfitFile = "msehead.txt" 

    galfitFile = path  + galfitFile
    #mgeargs.image=path+mgeargs.image
    #mgeargs.Ds9regFile=path+mgeargs.Ds9regFile
 
    regfile  = "ds9.mge2galfit.reg"
    regfile = path + regfile
    center = None
    psf = 0
    twist = None
    gauss = False 
    freeser = False 
    freesky = False 
    numgauss = None


    mge2gal(galfitFile, regfile, center, psf, twist, gauss, freeser, freesky, numgauss) 


    parfile="mseGALFIT.txt"
    file2  = "mgegas.txt"
    namefile="sectors.png"
    nameout="tests/ngc3344J.mge2galfit.png"
    namecons="constraints.txt"

    assert os.path.isfile(parfile)
    if os.path.isfile(parfile):
        os.remove(parfile)

    assert os.path.isfile(file2)
    if os.path.isfile(file2):
        os.remove(file2)

    assert os.path.isfile(namefile)
    if os.path.isfile(namefile):
        os.remove(namefile)

    assert os.path.isfile(nameout)
    if os.path.isfile(nameout):
        os.remove(nameout)

    assert os.path.isfile(namecons)
    if os.path.isfile(namecons):
        os.remove(namecons)




    return None




def test_sbProf():

    class sb_options:
        pass


    sbargs = sb_options()

    sbargs.Image  =  "A1656-1-3.sbProf.fits"      
    sbargs.Ds9Region = "ds9.sbProf.reg"
    path="tests/"
    sbargs.Image=path+sbargs.Image
    sbargs.Ds9Region=path+sbargs.Ds9Region
 


    sbargs.mgzpt = 21.817
    sbargs.mask = "mask.sbProf.fits"
    
    sbargs.mask = path+sbargs.mask

    sbargs.axrat = None 
    sbargs.angle = None 
    sbargs.sky = 376 
    sbargs.plate = 0.68 
    sbargs.output = "sb.png"
    sbargs.output= path+sbargs.output
    sbargs.center = False  
    sbargs.ranx = None 
    sbargs.rany = None 
    sbargs.logx = False 
    sbargs.pix = False  
    sbargs.grid = False 

    sbargs.rad = None 
    sbargs.rad2 = None 



    sbProf(sbargs)


    assert os.path.isfile(sbargs.output)

    if os.path.isfile(sbargs.output):
        os.remove(sbargs.output)


    namefile = "gal.png"
    #namefile = path+namefile

    assert os.path.isfile(namefile)

    if os.path.isfile(namefile):
        os.remove(namefile)



    return None


#####################
#####################
#####################
#####################





#! /usr/bin/env python

import numpy as np
import argparse
import os
from astropy.io import fits
import subprocess as sp
import scipy
import sys
import copy

from scipy.special import gamma, gammainc, gammaincinv

from scipy.optimize import bisect

import matplotlib.pyplot as plt


from galfitools.galin.galfit import Galfit, conver2Sersic, SelectGal, numComps, GetRadAng
from galfitools.galin.galfit import GalComps, GalHead




def getMissLight(galfitFile1, galfitFile2, dis, num_comp, rad):
    '''gets the missing light from the difference between two GALFIT models'''

    #reading arguments parsing

    galfit1 = Galfit(galfitFile1)
    head1 = galfit1.ReadHead()
    galcomps1 = galfit1.ReadComps()


    galfit2 = Galfit(galfitFile2)
    head2 = galfit2.ReadHead()
    galcomps2 = galfit2.ReadComps()



    galcomps1 = SelectGal(galcomps1, dis, num_comp)
    galcomps2 = SelectGal(galcomps2, dis, num_comp)



    #taking the last component position angle for the whole galaxy

    maskgal1 = (galcomps1.Active == True) 
    maskgal2 = (galcomps2.Active == True) 


    #convert all exp, gaussian and de vaucouleurs to Sersic format
    comps1 = conver2Sersic(galcomps1) 
    comps2 = conver2Sersic(galcomps2) 


    N1 = numComps(comps1,'all')
    N2 = numComps(comps2,'all')

    if (N1 == 0) or (N2 == 0):
        print('not enough number of components of one of the two models to proceed')
        print('exiting..')
        sys.exit(1)


    Flux1, mag1 = GetMag().GetFlux(head1, comps1, rad)
    Flux2, mag2 = GetMag().GetFlux(head2, comps2, rad)


    magmiss = getMiss(head1, mag1, mag2)




    return magmiss, N1, N2




class GetMag:


    def GetFlux(self, head, comps, gamRad):


        #comps.Rad = comps.Rad*head.scale
        comps.Flux = 10**((head.mgzpt - comps.Mag)/2.5)



        k = gammaincinv(2*comps.Exp, 0.5)

        denom1 = (2*np.pi*comps.Rad**2)*(np.exp(k))
        denom2 = (comps.Exp)*(k**(-2*comps.Exp))
        #axis ratio must be considered if mag total will be used:
        denom3 = (gamma(2*comps.Exp))*(comps.AxRat) 
        #denom3 = (gamma(2*comps.Exp))

        denom = denom1*denom2*denom3 
        
        comps.Ie = comps.Flux/denom

        maskgal = (comps.Active == True) 


        Ftotr = self.Ftotser(gamRad, comps.Ie[maskgal], comps.Rad[maskgal], comps.Exp[maskgal], comps.AxRat[maskgal], comps.PosAng[maskgal]) 

        mag = -2.5*np.log10(Ftotr) + head.mgzpt


        return Ftotr, mag 

     
    def Ftotser(self, R: float, Ie: list, rad: list, n: list, q: list, pa: list) -> float:

        FtotR = self.Fser(R, Ie, rad, n, q, pa ) 

        return FtotR.sum()



    def Fser(self, R: float, Ie: list, Re: list, n: list, q: list, pa: list) -> float:
        '''sersic flux to a determined R'''


        k = gammaincinv(2*n, 0.5)

        #Rcor = GetRadAng(R, q, pa) 
        x = k * (R/Re)**(1/n)

        Fr = Ie*(Re**2)*(2*np.pi*q*n)*np.exp(k)*(k**(-2*n))*gammainc(2*n, x)*gamma(2*n)




        
        return Fr





def getMiss(head, mag1, mag2):


    Flux1 = 10**((head.mgzpt - mag1)/2.5)
    Flux2 = 10**((head.mgzpt - mag2)/2.5)

    if Flux1 > Flux2:

        Fluxmiss = Flux1 - Flux2
    else:
        print("Warning: Flux1 is less than Flux2 for the selected radius. Inverting")

        Fluxmiss = Flux2 - Flux1



    magMiss = -2.5*np.log10(Fluxmiss) + head.mgzpt

    return magMiss






#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  main()






#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt
from astropy.io import fits
from os import path

import mgefit
from mgefit.mge_fit_1d import mge_fit_1d
from scipy.special import gammaincinv


from scipy.interpolate import UnivariateSpline

def main():



    rb = 10 

    print("computing mean of Rbreak/Rgamma")

    mean, dev, mi,ma = Rgamean(rb)

    print("mean = {:.2f} ; std = {:.2f} ".format(mean,dev))
    print("min value = {:.2f} ; max = {:.2f}".format(mi,ma))


def Rgamma(Rb,alpha,beta,gamma):


    rgam = Rb*(((0.5-gamma)/(beta-0.5))**(1/alpha)) #formula

    if np.isscalar(rgam):
        Rg  = rgam
    else:
        mask = np.isfinite(rgam)
        Rg  = rgam[mask]

    return Rg 



def Rgamean(Rb):

    alpha = np.arange(2,4.1,0.01) #range of alpha to search
    beta = np.arange(0.5,3.1,0.01) #range of beta to search
    gamma = np.arange(0,0.5,0.01) #range of gamma to search

    alpha = np.arange(1,4.1,0.01) #range of alpha to search
    beta = np.arange(0.5,2.1,0.01) #range of beta to search
    gamma = np.arange(0,0.5,0.01) #range of gamma to search



    albega = cartesian((alpha,beta,gamma))
    
    alpha= albega[:,0]
    beta = albega[:,1]
    gamma = albega[:,2]

    Rg = Rgamma(Rb,alpha,beta,gamma)

    #mask = np.isfinite(Rg)
    #output[~np.isfinite(output)] = 0

    kons=Rb/Rg

    kons.sort()

    tot=len(kons)
    
    print("removing top 80% and bottom 20%")

    top=round(.8*tot)
    bot=round(.2*tot)
    #top=np.int(top)
    #bot=np.int(bot)

    kon=kons[bot:top]

    n = len(kon)

    print("the total number of parameters combinations is",n)

    if n%2 == 0:
        idmed1 = int(n/2 - 1)
        idmed2 = int(n/2) 
        median = (kon[idmed1] + kon[idmed2])/2 
    else:
        idmed = int((n+1)/2 - 1)
        median = kon[idmed]

    print("the median is {:.2f} ".format(median))

    m = np.mean(kon)
    s = np.std(kon) 
    mi = np.min(kon)
    ma = np.max(kon)

    plt.close()
    plt.clf()

    counts, bins = np.histogram(kon)
    plt.stairs(counts, bins)
    plt.hist(bins[:-1], bins, weights=counts)
    #counts,bins,patches=plt.hist(kon)

    idmod = np.where(counts==max(counts))[0][0] 

    print("the mode is {:.2f} ".format(bins[idmod]))

    #plt.hist(kon)


    plt.grid(True)
    plt.minorticks_on()
    plt.savefig("histn.png")
 


    return m,s,mi,ma







def cartesian(arrays, out=None):
    """
    Generate a Cartesian product of input arrays.

    Parameters
    ----------
    arrays : list of array-like
        1-D arrays to form the Cartesian product of.
    out : ndarray
        Array to place the Cartesian product in.

    Returns
    -------
    out : ndarray
        2-D array of shape (M, len(arrays)) containing Cartesian products
        formed of input arrays.

    Examples
    --------
    >>> cartesian(([1, 2, 3], [4, 5], [6, 7]))
    array([[1, 4, 6],
           [1, 4, 7],
           [1, 5, 6],
           [1, 5, 7],
           [2, 4, 6],
           [2, 4, 7],
           [2, 5, 6],
           [2, 5, 7],
           [3, 4, 6],
           [3, 4, 7],
           [3, 5, 6],
           [3, 5, 7]])

    """

    arrays = [np.asarray(x) for x in arrays]
    dtype = arrays[0].dtype

    n = np.prod([x.size for x in arrays])
    if out is None:
        out = np.zeros([n, len(arrays)], dtype=dtype)

    #m = n / arrays[0].size
    m = int(n / arrays[0].size)
    out[:,0] = np.repeat(arrays[0], m)
    if arrays[1:]:
        cartesian(arrays[1:], out=out[0:m, 1:])
        for j in range(1, arrays[0].size):
        #for j in xrange(1, arrays[0].size):
            out[j*m:(j+1)*m, 1:] = out[0:m, 1:]
    return out



#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
    main()





import argparse


from galfitools.mge.mge2galfit import mge2gal
from galfitools.mge.SbProf import sbProf

from galfitools.shell.prt import printWelcome
#check modify: remove initgauss
def mainMGE():

    printWelcome()
    parser = argparse.ArgumentParser(description="fits mge of Cappellari and formats to GALFIT")

    #parser.add_argument("image", help="the Mask image file to modify or create")
    parser.add_argument("GalfitFile", help="GALFIT file to obtain the header options")
    parser.add_argument("Ds9regFile", help="the DS9 ellipse region file containing the galaxy. Ignored if -xy option is used")
    parser.add_argument("-t","--twist", action="store_true", help="uses twist option for mge ")
    parser.add_argument("-c","--center", action="store_true", 
                        help="uses the center given in DS9 region file, " + 
                        "otherwise it will found the x,y peaks within DS9 ellipse")
    parser.add_argument("-p","--psf", type=float, help="the value of PSF sigma ",default=0)
    parser.add_argument("-gas","--gauss", action="store_true", help="uses gauss function for galfit file")

    parser.add_argument("-fser","--freeser", action="store_true", help="leaves the sersic index as a free parameter to fit")
    parser.add_argument("-fsk","--freesky", action="store_true", help="leaves the sky as a free parameter to fit")


    parser.add_argument("-ng","--numgauss", type=int, help="number of gaussians that will be used for galfit. ")


    parser.add_argument("-xy","--xypos", nargs=2, type=int, help="provides the (x y) position center of the object to fit")


    parser.add_argument("-e","--ellip", type=float, help="ellipticity of object")
    parser.add_argument("-pa","--posang", type=float, help="position angle of object. Measured from Y-axis")


    args = parser.parse_args()



    galfitFile = args.GalfitFile
    regfile = args.Ds9regFile 
    twist = args.twist
    #regu = args.regu
    center = args.center
    psf = args.psf
    gauss = args.gauss

    #psfile = args.psfile
    #sigfile = args.sigfile

    freeser = args.freeser
    freesky = args.freesky

    numgauss = args.numgauss 

    xypos = args.xypos
    ellip = args.ellip
    posang = args.posang



    #mge2gal(args) 

    mge2gal(galfitFile, regfile, center, psf, twist, gauss, freeser, freesky, numgauss, xypos=xypos, ellip = ellip , posang = posang) 



def mainSbProf():

    printWelcome()
    parser = argparse.ArgumentParser(description = "SbProf: creates a surface brightness profile from a ellipse ds9 region")


    # required arguments
    parser.add_argument("Image", help = "image fits file")
    parser.add_argument("Ds9Region", help = "Ds9 ellipse region file")


    parser.add_argument("-q","--axrat", type=float, help="axis ratio")
    parser.add_argument("-pa","--angle", type=float, help="angular position (same as GALFIT)")

    parser.add_argument("-mz","--mgzpt", type=float, help="Magnitud zero point", default=25)
    parser.add_argument("-m","--mask", type=str, help="mask fits file" )

    parser.add_argument("-s","--sky", type=float, help="sky value. Default = 0", default=0)
    parser.add_argument("-p","--plate", type=float, help="plate scale ", default=1)
    parser.add_argument("-o","--output", type=str, help="output file", default="sb.png")

    parser.add_argument("-c","--center", action="store_true", 
                        help="uses the center given in DS9 region file," + 
                        "otherwise it will found the x,y peaks within DS9 ellipse")

    parser.add_argument("-rx","--ranx",nargs=2, type=float, help="provide a range for x-axis: xmin - xmax ")
    parser.add_argument("-ry","--rany", nargs=2,type=float, help="provide a range for y-axis: ymin - ymax  ")
 
    parser.add_argument("-lx","--logx", action="store_true", help="turn the X-axis to logarithm ")

    parser.add_argument("-px","--pix", action="store_true", help="turn the top x-axis in pixels ")
    parser.add_argument("-g","--grid", action="store_true", help="display a grid in the plot ")
 
    parser.add_argument("-r","--rad", type=float, help="value for a vertical line to add into the plot")
    parser.add_argument("-r2","--rad2", type=float, help="value for a second vertical line to add into the plot")

    args = parser.parse_args()


    sbProf(args)


    
    print('Done')








import argparse

from galfitools.sim.MakeSim import makeSim

from galfitools.shell.prt import printWelcome

def mainMakeSim():

    printWelcome()

    parser = argparse.ArgumentParser(description="simulates a observed galaxy from a GALFIT model")

    parser.add_argument("image", help="the GALFIT galaxy model")
    parser.add_argument("newimage", help="the name of the new galaxy image")


    parser.add_argument("-s","--sky", type=float, help="the sky background value. default = 0", default=0)
    parser.add_argument("-std","--std", type=float, help="the sky standard deviation. default = 1", default=1)

    parser.add_argument("-g","--gain", type=float, help="the gain value of the image. default =  1", default=1)



    args = parser.parse_args()

    image = args.image 
    GAIN = args.gain 

    skymean = args.sky
    skystd = args.std 

    newimage = args.newimage 




    makeSim(image, GAIN, skymean, skystd, newimage)









#! /usr/bin/env python

import numpy as np
import argparse
import os
from astropy.io import fits
import subprocess as sp
import scipy
import sys
import copy

from scipy.special import gamma, gammainc, gammaincinv

from scipy.optimize import bisect

import matplotlib.pyplot as plt


from galfitools.galin.galfit import Galfit, conver2Sersic, SelectGal, numComps,GetRadAng

from galfitools.galout.getRads import GetReff, GetMe



from galfitools.galin.galfit import GalComps, GalHead





def getCOW(galfitFile: str, dis: int, angle: float, frac: float,
        num_comp: int, plotname: str, dpival: int, galfitF2: str, maxdiff: bool) -> float:
    '''plots the curve-of-growth from the galfit.XX file. Only for Sersic functions'''


    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()
    galcomps = galfit.ReadComps()



    galcomps = SelectGal(galcomps, dis, num_comp)


    #taking the last component position angle for the whole galaxy
    maskgal = (galcomps.Active == True) 
    if angle:
        theta = angle
    else:
        theta = galcomps.PosAng[maskgal][-1]  



    #convert all exp, gaussian and de vaucouleurs to Sersic format
    comps = conver2Sersic(galcomps) 


    N = numComps(comps,'all')
    #print('number of model components: ',N)

    if N == 0:
        print('not enough number of components to plot COW')
        print('exiting..')
        sys.exit(1)

    line = 'Using a theta value of : {:.2f} degrees \n'.format(theta)
    #print(line)



    EffRadfrac, totmag = GetReff().GetReSer(head, comps, frac, theta) #obtains the radius at 95% of total flux


    #computing the Sersic indexes for different radius

    R = np.arange(0.1, EffRadfrac,.1) 

    cows, totmag = GetCOW().GetCOWrad(head, comps, theta, R)

    if galfitF2:

        #it uses the same num_comp as the first file
        head2, comps2, theta2 = readGalfitF2(galfitF2, dis, angle, num_comp) 


        cows2, totmag2 = GetCOW().GetCOWrad(head2, comps2, theta, R) #same theta as galfitF1


        diff =  np.abs(cows - cows2)

        idx = np.where(diff == np.max(diff))


        xline = R[idx][0]


        ymin = cows[idx] 
        ymax = cows2[idx]




    plt.plot(R, cows, color='blue', label='model 1', dpi=dpival)
    plt.xlabel("Rad")
    plt.ylabel("curve of growth")
    plt.grid(True)
    plt.minorticks_on()

    xmin = 0.1 
    xmax = np.max(R) 
   


    plt.gca().invert_yaxis()
    
    plt.xlabel("Radius (pixels)")
    plt.title("Curve of Growth ")
    plt.ylabel("magnitude (< R) ")


    
    if galfitF2:

        plt.plot(R, cows2, color='green',label='model 2', dpi=dpival)

        plt.xlabel("Rad")
        plt.ylabel("curve of growth")

        if maxdiff:
            plt.vlines(xline, ymin, ymax, color='red',label='max diff')


    plt.hlines(totmag, xmin, xmax, color='black',label='total magnitude')
    
    plt.legend(loc='lower right')
    plt.savefig(plotname)

    return totmag, N, theta 



def readGalfitF2(galfitF2, dis, angle, num_comp):

    galfit = Galfit(galfitF2)

    head2 = galfit.ReadHead()
    galcomps2 = galfit.ReadComps()


    galcomps2 = SelectGal(galcomps2, dis, num_comp)


    #taking the last component position angle for the whole galaxy
    maskgal = (galcomps2.Active == True) 
    if angle:
        theta2 = angle
    else:
        theta2 = galcomps2.PosAng[maskgal][-1]  

    #convert all exp, gaussian and de vaucouleurs to Sersic format
    comps2 = conver2Sersic(galcomps2) 


    N = numComps(comps2,'all')
    #print('number of model components: ',N)

    if N == 0:
        print('not enough number of components to plot COW')
        print('exiting..')
        sys.exit(1)

    line = 'Using a theta value of : {:.2f} degrees \n'.format(theta2)
    #print(line)


    return head2, comps2, theta2 


### GetCOW
class GetCOW:
    '''class to obtain the Curve-of-Growth at a given radius '''


    def GetCOWrad(self, head, comps, theta, R):


        cowfluxs = np.array([])

        for r in R: #check if functions works without the for 

            cowflux = self.GetCOWFluxR(head, comps, theta, r)

            cowfluxs = np.append(cowfluxs, cowflux)



        maskgal = (comps.Active == True) 

        comps.Flux = 10**((head.mgzpt - comps.Mag)/2.5)

        totFlux = comps.Flux[maskgal].sum()

        totmag = -2.5*np.log10(totFlux) + head.mgzpt


        cows = -2.5*np.log10(cowfluxs) + head.mgzpt


        return cows, totmag 




    def GetCOWFluxR(self, galhead: GalHead, comps: GalComps, theta: float, rad: float) -> float:


        maskgal = (comps.Active == True) 

        comps.Flux = 10**((galhead.mgzpt - comps.Mag)/2.5)

        totFlux = comps.Flux[maskgal].sum()

        totmag = -2.5*np.log10(totFlux) + galhead.mgzpt

        #rad refers to the radius where flux will be computed
        #comps.Rad refers to the effective radius of every component


        cowR = self.funCOWSer(rad, comps.Flux[maskgal], comps.Rad[maskgal], 
                comps.Exp[maskgal], comps.AxRat[maskgal], comps.PosAng[maskgal],  theta)



        return cowR


    def funCOWSer(self, R: float, flux: list, rad: list, n: list, q: list, pa: list, theta: float) -> float:
        

        fun = self.COWtotser(R, flux, rad, n, q, pa, theta) #- totFlux*eff

        return fun
    

    def COWtotser(self, R: float, flux: list, rad: list, n: list, q: list, pa: list, theta: float) -> float:

        ftotR = self.COWser(R, flux, rad, n, q, pa, theta) 

        return ftotR.sum()



    def COWser(self, R: float, Flux: list, Re: list, n: list, q: list, pa: list, theta: float) -> float:
        '''sersic flux to a determined R'''
        
        k = gammaincinv(2*n, 0.5)

        Rcor = GetRadAng(R, q, pa, theta) 

        X = k*(Rcor/Re)**(1/n) 

        Fr = Flux*gammainc(2*n, X) 
        
        return Fr


############################
############################
#





class GetN:
    '''Class to compute the Sersic index from photometric parameters'''



    def GalNs(self, EffRad, EffRadfrac, F):



        sers = np.array([])

        for idx, f in  enumerate(F):


            n = self.ReRfrac(EffRad, EffRadfrac[idx], f)

            sers = np.append(sers, n)



        return sers 





    def MeMeanMe(self, me: float, meanme: float) -> float:

        a = 0.1
        b = 12


        Sersic = self.solveSer(a, b, me, meanme)


        return Sersic



    def solveSer(self, a: float, b: float, me: float, meanme: float) -> float:
        "return the sersic index. It uses Bisection"

        try:

            N = bisect(self.funMeMeanMe, a, b, args=(me, meanme))

        except:
            print("solution not found for the given range")
            N = 0

        return N 


    def funMeMeanMe(self, n: float, me: float, meanme: float) -> float:
        

        k = gammaincinv(2*n, 0.5)

        fn = self.Fn(n)

        result = me - meanme  - 2.5*np.log10(fn)


        return result 
     
    def Fn(self, n: float) -> float:

        k = gammaincinv(2*n, 0.5)


        fn = ((n*np.exp(k))/(k**(2*n)))*gamma(2*n)


        return fn


    def ReRfrac(self, Re: float, Rfrac: float, frac: float) -> float:

        a = 0.1
        b = 12

        Sersic = self.solveSerRe(a, b, Re, Rfrac, frac)


        return Sersic



    def solveSerRe(self, a: float, b: float, Re: float, Rfrac: float, frac: float) -> float:
        "return the sersic index. It uses Bisection"

        try:
            N = bisect(self.funReRfrac, a, b, args=(Re, Rfrac, frac))

        except:
            print("solution not found for the given range")
            N = 0



        return N 


    def funReRfrac(self, n: float, Re: float, Rfrac: float, frac: float) -> float:
        

        k = gammaincinv(2*n, 0.5)


        x = k*(Rfrac/Re)**(1/n)
        

        result = frac*gamma(2*n) - gamma(2*n)*gammainc(2*n, x) 


        return result 
     


    def MeM0(self, me: float, m0: float) -> float:
        '''Uses me and m0 to compute n. It is not very realiable 
        and takes longer than the other two methods'''
        a = 0
        b = 45


        K = self.solveKm0(a, b, me, m0)


        a = 0.2
        b = 40



        Sersic = self.solveSerK(a, b, K)


        return Sersic

    def solveKm0(self, a: float, b: float, me: float, m0: float) -> float:
        "return the sersic index. It uses Bisection"

        try:
            K = bisect(self.funMeM0, a, b, args=(me, m0))

        except:
            print("solution not found for the given range")
            K = 0



        return K 


    def funMeM0(self, K: float, me: float, m0: float) -> float:

        result = me - m0 - 2.5*K/np.log(10)

        return result 

    def solveSerK(self, a: float, b: float, k: float) -> float: 

        try:
            sersic = bisect(self.funK, a, b, args=(k))

        except:
            print("solution not found for the given range")
            sersic = 0



        return sersic

    def funK(self, n: float, k: float) -> float:
       

        result = gammaincinv(2*n, 0.5) - k



        return result 




#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  main()






#!/usr/bin/env python3


import numpy as np
from astropy.io import fits
import sys
import subprocess as sp
import os.path


from galfitools.galout.getPeak import getPeak
from galfitools.galout.PhotDs9 import photDs9 

from galfitools.mge.mge2galfit import GetInfoEllip
from galfitools.galin.galfit import GalComps, GalHead

def getSersic(image: float, regfile: str, center: bool, maskfile: str, 
                zeropoint: float, sky: float, noprint: float, 
                bulgetot: float, bards9: str) -> GalComps:


    X, Y, AxRat, PA = getPeak(image, regfile, center, maskfile)

    mag, exptime = photDs9(image, regfile, maskfile, zeropoint, sky)


    obj, xpos, ypos, rx, ry, angle = GetInfoEllip(regfile)


    if bards9:

        Xbar, Ybar, AxRatbar, PAbar = getPeak(image, bards9, center, maskfile)
        magbar, exptimebar = photDs9(image, bards9, maskfile ,zeropoint, sky)
        objbar, xposbar, yposbar, rxbar, rybar, anglebar = GetInfoEllip(bards9)





    if bulgetot:
        Fluxtot = 10**(-mag/2.5)
        FluxBulge = Fluxtot*bulgetot
        FluxDisk = Fluxtot - FluxBulge
        mag = -2.5*np.log10(FluxBulge)
        mag2 = -2.5*np.log10(FluxDisk)

        if bards9:
            Fluxbar = 10**(-magbar/2.5) #assuming bar flux containts bulge flux
            FluxDisk = Fluxtot - Fluxbar
            FluxBulge = Fluxbar*.3   #wild guess
            Fluxbar = Fluxbar*.7   #wild guess
            mag = -2.5*np.log10(FluxBulge)
            mag2 = -2.5*np.log10(FluxDisk)
            magbar = -2.5*np.log10(Fluxbar)

            if rxbar >= rybar:
                Rebar = rxbar/2 #wild guess
            else:
                Rebar = rybar/2 #same



    if rx >= ry:
        Re = rx/2 #wild guess
    else:
        Re = ry/2 #same

    #wild guesses for n and Re
    n = 2
    skip = 0


    N = 1
    #store in GalComps data class
    galcomps = GalComps()


    if (bulgetot): 
        if not(noprint):
            print("The initial parameters for the Sersic component based on the DS9 ellipse region are: ")
            print("")
            print("WARNING: these are initial parameters. True values will be computed by GALFIT")
            print("")


            print("# Component number: 1")
            print("0) sersic # Component type")
            print("1) {:.2f} {:.2f} 1 1   # Position x, y".format(X, Y))
            print("3) {:.2f}   1       # Integrated magnitude ".format(mag))
            print("4) {:.2f}   1       # R_e (effective radius) ".format(Re*bulgetot)) #wild guess
            print("5) {:.2f}   1       # Sersic index n  ".format(n))
            print("6) 0.0000   0      # ----  ")
            print("7) 0.0000   0      # ----  ")
            print("8) 0.0000   0      # ----  ")
            print("9) {:.2f}   1       # Axis Ratio (b/a)  ".format(1))
            print("10) {:.2f}  1       # Position angle (PA)  ".format(0))
            print("Z) {}  # Skip this model in output image?  ".format(skip))

            print("")

            if bards9:
                print("# Component number: 1b bar")
                print("0) sersic # Component type")
                print("1) {:.2f} {:.2f} 1 1   # Position x, y".format(X, Y))
                print("3) {:.2f}    1      # Integrated magnitude ".format(magbar))
                print("4) {:.2f}    1      # R_e (effective radius) ".format(Rebar))
                print("5) {:.2f}    0      # Sersic index n  ".format(0.5))
                print("6) 0.0000   0      # ----  ")
                print("7) 0.0000   0      # ----  ")
                print("8) 0.0000   0      # ----  ")
                print("9) {:.2f}   1       # Axis Ratio (b/a)  ".format(AxRatbar))
                print("10) {:.2f}  1       # Position angle (PA)  ".format(PAbar))
                print("Z) {}  # Skip this model in output image?  ".format(skip))
                print("")


            print("# Component number: 2")
            print("0) sersic # Component type")
            print("1) {:.2f} {:.2f} 1 1   # Position x, y".format(X, Y))
            print("3) {:.2f}    1      # Integrated magnitude ".format(mag2))
            print("4) {:.2f}    1      # R_e (effective radius) ".format(Re))
            print("5) {:.2f}    0      # Sersic index n  ".format(1))
            print("6) 0.0000   0      # ----  ")
            print("7) 0.0000   0      # ----  ")
            print("8) 0.0000   0      # ----  ")
            print("9) {:.2f}    1      # Axis Ratio (b/a)  ".format(AxRat))
            print("10) {:.2f}   1      # Position angle (PA)  ".format(PA))
            print("Z) {}  # Skip this model in output image?  ".format(skip))



            print("")
            print("for parameter constraints file: ") 
            if bards9:

                print(" 1_2_3    x    offset ")
                print(" 1_2_3    y    offset ")
            else:

                print(" 1_2    x    offset ")
                print(" 1_2    y    offset ")
 
        #first component: bulge
        galcomps.PosX = np.append(galcomps.PosX, X)
        galcomps.PosY = np.append(galcomps.PosY, Y)
        galcomps.NameComp = np.append(galcomps.NameComp, "sersic")
        galcomps.N = np.append(galcomps.N, N)
                        
        galcomps.Mag = np.append(galcomps.Mag, mag)
        galcomps.Rad = np.append(galcomps.Rad, Re*bulgetot)
        galcomps.Exp = np.append(galcomps.Exp, n)
        galcomps.Exp2 = np.append(galcomps.Exp2, 0)
        galcomps.Exp3 = np.append(galcomps.Exp3, 0)
        galcomps.AxRat = np.append(galcomps.AxRat, 1)
        galcomps.PosAng = np.append(galcomps.PosAng, 0)
        galcomps.skip = np.append(galcomps.skip, skip)

        #free parameters
        galcomps.PosXFree = np.append(galcomps.PosXFree, 1)
        galcomps.PosYFree = np.append(galcomps.PosYFree, 1)
        galcomps.MagFree = np.append(galcomps.MagFree, 1)
        galcomps.RadFree = np.append(galcomps.RadFree, 1)
        galcomps.ExpFree = np.append(galcomps.ExpFree, 1)
        galcomps.Exp2Free = np.append(galcomps.Exp2Free, 0)
        galcomps.Exp3Free = np.append(galcomps.Exp3Free, 0)
        galcomps.AxRatFree = np.append(galcomps.AxRatFree, 1)
 


        if bards9:

            #alternative component: bar
            N = N + 1
            galcomps.PosX = np.append(galcomps.PosX, X)
            galcomps.PosY = np.append(galcomps.PosY, Y)
            galcomps.NameComp = np.append(galcomps.NameComp, "sersic")
            galcomps.N = np.append(galcomps.N, N)
                            
            galcomps.Mag = np.append(galcomps.Mag, magbar)
            galcomps.Rad = np.append(galcomps.Rad, Rebar)
            galcomps.Exp = np.append(galcomps.Exp, 0.5)
            galcomps.Exp2 = np.append(galcomps.Exp2, 0)
            galcomps.Exp3 = np.append(galcomps.Exp3, 0)
            galcomps.AxRat = np.append(galcomps.AxRat, AxRatbar)
            galcomps.PosAng = np.append(galcomps.PosAng, PAbar)
            galcomps.skip = np.append(galcomps.skip, skip)

            #free parameters
            galcomps.PosXFree = np.append(galcomps.PosXFree, 1)
            galcomps.PosYFree = np.append(galcomps.PosYFree, 1)
            galcomps.MagFree = np.append(galcomps.MagFree, 1)
            galcomps.RadFree = np.append(galcomps.RadFree, 1)
            galcomps.ExpFree = np.append(galcomps.ExpFree, 0)
            galcomps.Exp2Free = np.append(galcomps.Exp2Free, 0)
            galcomps.Exp3Free = np.append(galcomps.Exp3Free, 0)
            galcomps.AxRatFree = np.append(galcomps.AxRatFree, 1)
     

        #second component: disk
        N = N + 1
        galcomps.PosX = np.append(galcomps.PosX, X)
        galcomps.PosY = np.append(galcomps.PosY, Y)
        galcomps.NameComp = np.append(galcomps.NameComp, "sersic")
        galcomps.N = np.append(galcomps.N, N)
                        
        galcomps.Mag = np.append(galcomps.Mag, mag2)
        galcomps.Rad = np.append(galcomps.Rad, Re)
        galcomps.Exp = np.append(galcomps.Exp, 1)
        galcomps.Exp2 = np.append(galcomps.Exp2, 0)
        galcomps.Exp3 = np.append(galcomps.Exp3, 0)
        galcomps.AxRat = np.append(galcomps.AxRat, AxRat)
        galcomps.PosAng = np.append(galcomps.PosAng, PA)
        galcomps.skip = np.append(galcomps.skip, skip)


        #free parameters
        galcomps.PosXFree = np.append(galcomps.PosXFree, 1)
        galcomps.PosYFree = np.append(galcomps.PosYFree, 1)
        galcomps.MagFree = np.append(galcomps.MagFree, 1)
        galcomps.RadFree = np.append(galcomps.RadFree, 1)
        galcomps.ExpFree = np.append(galcomps.ExpFree, 0)
        galcomps.Exp2Free = np.append(galcomps.Exp2Free, 0)
        galcomps.Exp3Free = np.append(galcomps.Exp3Free, 0)
        galcomps.AxRatFree = np.append(galcomps.AxRatFree, 1)
 


    else:

        if not(noprint):
            print("The initial parameters for the Sersic component based on the DS9 ellipse region are: ")
            print("")
            print("WARNING: these are initial parameters. True values will be computed by GALFIT")
            print("")

            print("# Component number: 1")
            print("0) sersic # Component type")
            print("1) {:.2f} {:.2f} 1 1  # Position x, y".format(X, Y))
            print("3) {:.2f}    1      # Integrated magnitude ".format(mag))
            print("4) {:.2f}    1      # R_e (effective radius) ".format(Re))
            print("5) {:.2f}    1      # Sersic index n  ".format(n))
            print("6) 0.0000   0      # ----  ")
            print("7) 0.0000   0      # ----  ")
            print("8) 0.0000   0      # ----  ")
            print("9) {:.2f}    1      # Axis Ratio (b/a)  ".format(AxRat))
            print("10) {:.2f}   1      # Position angle (PA)  ".format(PA))
            print("Z) {}  # Skip this model in output image?  ".format(skip))



        galcomps.PosX = np.append(galcomps.PosX, X)
        galcomps.PosY = np.append(galcomps.PosY, Y)
        galcomps.NameComp = np.append(galcomps.NameComp, "sersic")
        galcomps.N = np.append(galcomps.N, N)
                        
        galcomps.Mag = np.append(galcomps.Mag, mag)
        galcomps.Rad = np.append(galcomps.Rad, Re)
        galcomps.Exp = np.append(galcomps.Exp, n)
        galcomps.Exp2 = np.append(galcomps.Exp2, 0)
        galcomps.Exp3 = np.append(galcomps.Exp3, 0)
        galcomps.AxRat = np.append(galcomps.AxRat, AxRat)
        galcomps.PosAng = np.append(galcomps.PosAng, PA)
        galcomps.skip = np.append(galcomps.skip, skip)

        #free parameters
        galcomps.PosXFree = np.append(galcomps.PosXFree, 1)
        galcomps.PosYFree = np.append(galcomps.PosYFree, 1)
        galcomps.MagFree = np.append(galcomps.MagFree, 1)
        galcomps.RadFree = np.append(galcomps.RadFree, 1)
        galcomps.ExpFree = np.append(galcomps.ExpFree, 1)
        galcomps.Exp2Free = np.append(galcomps.Exp2Free, 0)
        galcomps.Exp3Free = np.append(galcomps.Exp3Free, 0)
        galcomps.AxRatFree = np.append(galcomps.AxRatFree, 1)
 


    return galcomps 






#!/usr/bin/env python3


import numpy as np
from astropy.io import fits
import sys
import subprocess as sp
import os.path


from galfitools.galin.MaskDs9 import GetAxis 
from galfitools.galin.MaskDs9 import checkCompHDU

def getStar(image: str, regfile: str, imsize: int, center: bool, sky: float, imout: str, sigma: str , sigout: str)-> None: 


    ##########################################
    ######Internal flags #####################
    even = False
    ##########################################
    ##########################################


    #root_ext = os.path.splitext(image)
    #timg= root_ext[0]

    mask = np.array([]) #empty for the moment

    i = 0 #index where data is located at hdu

    if(checkCompHDU(image)):
        i=1

    hdu = fits.open(image)
    img = hdu[i].data

    #img = img.astype(float)

    hdu.close()
    
    (ncol, nrow) = GetAxis(image)
    
    obj, xpos, ypos, rx, ry, angle = GetInfoEllip(regfile)
    xx, yy, Rkron, theta, eps = Ds9ell2Kronell(xpos,ypos,rx,ry,angle)

    if center:
        print('center of ds9 ellipse region will be used')
        xpeak, ypeak = round(xpos), round(ypos)
    else:        
        (xmin, xmax, ymin, ymax) = GetSize(xx, yy, Rkron, theta, eps, ncol, nrow)
        xpeak, ypeak = GetPmax(img, mask, xmin, xmax, ymin, ymax)
 
    print("object found at ", xpeak + 1, ypeak + 1)
    print("Ellipticity, Angle = ", eps, theta)

    if sky:
        print("Sky = ", sky)
    else:
        print("No sky input")



    if imsize % 2 == 0:
        even = True

    
    if even:
        lx = int(imsize/2)

        xlo = xpeak - lx + 1 
        ylo = ypeak - lx + 1 

        xhi = xpeak + lx 
        yhi = ypeak + lx
        
    else:
        lx = int(imsize/2 + 0.5)

        xlo = xpeak - lx + 2 
        ylo = ypeak - lx + 2

        xhi = xpeak + lx 
        yhi = ypeak + lx 
 



    GetFits(image, imout, sky, xlo, xhi, ylo, yhi)

    if sigma:
        GetFits(sigma, sigout, 0, xlo, xhi, ylo, yhi)



####################################################
####################################################
#####################################################



def GetFits(Image, Imageout, sky, xlo, xhi, ylo, yhi):
    "Get a piece from the image"
    # k Check


    #if os.path.isfile(Imageout):
    #    print("{} deleted; a new one is created \n".format(Imageout))
    #    runcmd = "rm {}".format(Imageout)
    #    errrm = sp.run([runcmd], shell=True, stdout=sp.PIPE,
    #                    stderr=sp.PIPE, universal_newlines=True)

    if os.path.isfile(Imageout):
        print("{} deleted; a new one is created \n".format(Imageout))
    #new fits


    i = 0 #index indicated where the data is located
    if(checkCompHDU(Image)):
        i=1
        newhdu = fits.CompImageHDU()
    else:
        newhdu = fits.PrimaryHDU()

    hdu = fits.open(Image)
    dat = hdu[i].data[ylo - 1:yhi, xlo - 1:xhi]
    head = hdu[i].header

    if sky:
        newhdu.data = dat - sky
    else:
        newhdu.data = dat 

    newhdu.header = head

    newhdu.writeto(Imageout, overwrite=True)

    hdu.close()



def GetInfoEllip(regfile):

    if not os.path.exists(regfile):
        print ('%s: reg filename does not exist!' %(regfile))
        sys.exit()

    f1 = open(regfile,'r')

    lines = f1.readlines()

    f1.close()

    flag = False
    found = False

    #reading reg file
    for line in lines:


        b1 = line.split("(")
        p = line.split(",")

        x1 = p[0]
        if b1[0] == "ellipse":

            x0 = "ellipse"
            x2 = x1[8:]
            flag = True
            found = True

        if (flag == True):
            x3 = p[4]
            x4 = x3[:-2]


            v0 = x0

            v1 = float(x2)
            v2 = float(p[1])
            v3 = float(p[2])
            v4 = float(p[3])
            v5 = float(x4)

            flag = False

    if found:
        obj  = v0
        xpos = v1
        ypos = v2
        rx = v3
        ry = v4
        angle = v5

        if rx >= ry:
            axratio = ry/rx 
            eps = 1 - axratio 
            theta = angle + 90 
        else:
            axratio = rx/ry 
            eps = 1 - axratio 
            theta = angle 

        return obj, xpos, ypos, rx, ry, angle
        #return eps, theta, xpos, ypos
    else:
        print("ellipse region was not found in file. Exiting.. ")
        sys.exit()

    return 0, 0, 0, 0



def Ds9ell2Kronell(xpos,ypos,rx,ry,angle):


    if rx >= ry:

        q = ry/rx
        e = 1 - q
        Rkron = rx
        theta = angle + 90
        xx = xpos
        yy = ypos
    else:
        q = rx/ry
        e = 1 - q
        Rkron = ry
        theta = angle# + 90
        xx = xpos
        yy = ypos
 
     
    return xx, yy, Rkron, theta, e 


def GetSize(x, y, R, theta, ell, ncol, nrow):
    "this subroutine get the maximun"
    "and minimim pixels for Kron and sky ellipse"
    # k Check
    q = (1 - ell)
    bim = q * R

    theta = theta * (np.pi / 180)  # rads!!

# getting size

    xmin = x - np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    xmax = x + np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    ymin = y - np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)

    ymax = y + np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)

    mask = xmin < 1
    if mask.any():
        xmin[mask] = 1

    mask = xmax > ncol
    if mask.any():
        xmax[mask] = ncol

    mask = ymin < 1
    if mask.any():
        ymin[mask] = 1

    mask = ymax > nrow
    if mask.any():
        ymax[mask] = nrow

    return (xmin, xmax, ymin, ymax)



def GetPmax(image, mask, xmin, xmax, ymin, ymax):


    xmin = int(xmin)
    xmax = int(xmax)
    ymin = int(ymin)
    ymax = int(ymax)

    chuckimg = image[ymin - 1:ymax, xmin - 1:xmax]
    if mask.any():
        chuckmsk = mask[ymin - 1:ymax, xmin - 1:xmax]

        invmask = np.logical_not(chuckmsk)

        invmask = invmask*1

        chuckimg = chuckimg*invmask

    maxy, maxx = np.where(chuckimg == np.max(chuckimg))
    
    xpos = maxx[0] + xmin - 1
    ypos = maxy[0] + ymin - 1

    return (xpos, ypos)



#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################

if __name__ == '__main__':
    mainGetStar()



#! /usr/bin/env python

import numpy as np
import argparse
import os
from astropy.io import fits
import subprocess as sp
import scipy
import sys
import copy

from scipy.special import gamma, gammainc, gammaincinv

from scipy.optimize import bisect

import matplotlib.pyplot as plt


from galfitools.galin.galfit import Galfit, conver2Sersic, SelectGal, numComps,GetRadAng

from galfitools.galout.getRads import GetReff, GetMe



from galfitools.galin.galfit import GalComps, GalHead





def getN(galfitFile: str, dis: int, frac: float, angle: float, 
            num_comp: int, plot: bool, const = 0) -> float:
    '''gets the effective radius from a set of Sersics'''



    eff = 0.5

    galfit = Galfit(galfitFile)

    head = galfit.ReadHead()
    galcomps = galfit.ReadComps()





    galcomps = SelectGal(galcomps, dis, num_comp)

    #taking the last component position angle for the whole galaxy

    maskgal = (galcomps.Active == True) 
    if angle:
        theta = angle
    else:
        theta = galcomps.PosAng[maskgal][-1]  

    #theta = 18.2534 


    #convert all exp, gaussian and de vaucouleurs to Sersic format
    comps = conver2Sersic(galcomps) 


    N = numComps(comps,'all')
    #print('number of model components: ',N)

    if N == 0:
        print('not enough number of components to compute Re')
        print('exiting..')
        sys.exit(1)

    line = 'Using a theta value of : {:.2f} degrees \n'.format(theta)
    #print(line)



    EffRad, totmag = GetReff().GetReSer(head, comps, eff, theta)

    line = 'Total Magnitude of the galaxy: {:.2f} \n'.format(totmag)
    #print(line)


    line = 'The radius at {:.0f}% of light is {:.2f} pixels \n'.format(eff*100,EffRad)
    #print(line)



    EffRadfrac, totmag = GetReff().GetReSer(head, comps, frac, theta)


    line = 'The radius at {:.0f}% of light is {:.2f} pixels \n'.format(frac*100, EffRadfrac)
    #print(line)



    comps2 =  copy.deepcopy(comps)
    meanme = GetMe().MeanMe(totmag, EffRad*head.scale)
    #me = GetMe().Me(head, comps2, 0, theta) #substituing effective radius per 0 gives m0
    me = GetMe().Me(head, comps2, EffRad*head.scale, theta) #substituing effective radius per 0 gives m0




    line = 'Mean Surface Brightness at effective radius: {:.2f} mag/\" \n'.format(meanme)
    #print(line)


    line = 'Surface brightness at effective radius {:.2f} mag/\" \n'.format(me)
    #print(line)

    sersic = GetN().MeMeanMe(me, meanme)

    line = 'Sersic index with the method of Mean Surface Brightness at effective radius: {:.2f}  \n'.format(sersic)
    #print(line)


    sersic2 = GetN().ReRfrac(EffRad, EffRadfrac, frac)

    line = 'Sersic index with the method of fraction of light at {:.2f}: {:.2f}  \n'.format(frac,sersic2)
    #print(line)


    #sersic3 = GetN().MeM0(me, m0)

    #line = 'Sersic index with the method of me - mo is {:.2f}  \n'.format(sersic3)
    #print(line)




    #computing the Sersic indexes for different radius

    Fa = np.arange(0.1, .5, .05)
    Fb = np.arange(.55,1,.05)
    F = np.concatenate((Fa,Fb))

    R = GetReff().GetRfracSer(head, comps, F, theta)


    ns = GetN().GalNs(EffRad, R, F) 


    if plot:

        plt.plot(F, ns - const)
        plt.grid(True)
        plt.minorticks_on()
        plt.xlabel("Fraction of light")
        plt.ylabel("Sersic index")
        plt.savefig("Serind.png")


    #print("Sersic index computed at different fraction of light radius: ")


    #for idx, item in enumerate(F):
    #    line = 'Fraction of light: {:.2f} ; Sersic index: {:.2f} '.format(F[idx],ns[idx])
    #    print(line)

    line = '\nSersic index mean: {:.2f}  Standard deviation: {:.2f}  '.format(np.mean(ns),np.std(ns))
    #print(line)


    
    #separate in two functions:

    #return Effrad, totmag, meanme, me, N, theta 
    return sersic, np.mean(ns), np.std(ns), totmag, N, theta 


class GetN:
    '''Class to compute the Sersic index from photometric parameters'''



    def GalNs(self, EffRad, EffRadfrac, F):



        sers = np.array([])

        for idx, f in  enumerate(F):


            n = self.ReRfrac(EffRad, EffRadfrac[idx], f)

            sers = np.append(sers, n)



        return sers 





    def MeMeanMe(self, me: float, meanme: float) -> float:

        a = 0.1
        b = 12

        Sersic = self.solveSer(a, b, me, meanme)


        return Sersic



    def solveSer(self, a: float, b: float, me: float, meanme: float) -> float:
        "return the sersic index. It uses Bisection"

        try:
            N = bisect(self.funMeMeanMe, a, b, args=(me, meanme))
        except:
            print("unable to solve equation in the given range. Setting n to 99")
            N = 99

        return N 


    def funMeMeanMe(self, n: float, me: float, meanme: float) -> float:
        

        k = gammaincinv(2*n, 0.5)

        fn = self.Fn(n)

        result = me - meanme  - 2.5*np.log10(fn)


        return result 
     
    def Fn(self, n: float) -> float:

        k = gammaincinv(2*n, 0.5)


        fn = ((n*np.exp(k))/(k**(2*n)))*gamma(2*n)


        return fn


    def ReRfrac(self, Re: float, Rfrac: float, frac: float) -> float:

        a = 0.1
        b = 12

        Sersic = self.solveSerRe(a, b, Re, Rfrac, frac)


        return Sersic



    def solveSerRe(self, a: float, b: float, Re: float, Rfrac: float, frac: float) -> float:
        "return the sersic index. It uses Bisection"

        try:
            N = bisect(self.funReRfrac, a, b, args=(Re, Rfrac, frac))
        except:
            print("unable to solve equation in the given range. Setting n to 99")
            N = 99



        return N 


    def funReRfrac(self, n: float, Re: float, Rfrac: float, frac: float) -> float:
        

        k = gammaincinv(2*n, 0.5)


        x = k*(Rfrac/Re)**(1/n)
        

        result = frac*gamma(2*n) - gamma(2*n)*gammainc(2*n, x) 


        return result 
     


    def MeM0(self, me: float, m0: float) -> float:
        '''Uses me and m0 to compute n. It is not very realiable 
        and takes longer than the other two methods'''
        a = 0
        b = 45


        K = self.solveKm0(a, b, me, m0)


        a = 0.2
        b = 40



        Sersic = self.solveSerK(a, b, K)


        return Sersic

    def solveKm0(self, a: float, b: float, me: float, m0: float) -> float:
        "return the sersic index. It uses Bisection"

        try:
            K = bisect(self.funMeM0, a, b, args=(me, m0))
        except:
            print("solution not found for the given range")
            K = 0

        return K 


    def funMeM0(self, K: float, me: float, m0: float) -> float:

        result = me - m0 - 2.5*K/np.log(10)

        return result 

    def solveSerK(self, a: float, b: float, k: float) -> float: 

        try:
            sersic = bisect(self.funK, a, b, args=(k))
        except:
            print("unable to solve equation in the given range. Setting n to 99")
            N = 99

        return sersic

    def funK(self, n: float, k: float) -> float:
       

        result = gammaincinv(2*n, 0.5) - k



        return result 




#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  main()






#! /usr/bin/python3


import numpy as np
import argparse
import glob

from pathlib import Path

# code to make init galfit files 



class InitGal:


    def __init__(self, GalfitFile: str, number: int, param3: float, param4: float, param5: float, param6: float, param7: float, param8: float, param9: float, param10: float, numcomp: int):

        self.GalfitFile = GalfitFile 
        self.number = number
        self.param3 = param3
        self.param4 = param4
        self.param5 = param5
        self.param6 = param6
        self.param7 = param7

        self.param8 = param8
        self.param9 = param9
        self.param10 = param10

        self.numcomp = numcomp

        self.MakeParams()
        self.MakeBash()




    def MakeParams(self):   



        lines = self.ReadFile(self.GalfitFile) #read  galfit input file

        (tmp)=self.GalfitFile.split(".")
        prefixname=tmp[0]


        for idx, item in enumerate(np.arange(1,self.number+1)): # cicle for every galfit file

            self.namefile = prefixname + "-" + str(item) + ".gal"

            fileout = open(self.namefile, "w")

            flagsky3 = False # flag to avoid to write in sky c3

            index = 0
            comp = 0


            while index < len(lines):

                line = lines[index]
                (tmp) = line.split()


                if (tmp[0] == "0)"):
                    comp = comp + 1

                if self.numcomp:

                    if self.numcomp == comp:

                        if (tmp[0] == "3)") and (flagsky3  == False):     # input image
                            if self.param3:
                                c3min, c3max = self.param3[0],self.param3[1]
                                c3 = np.random.default_rng().uniform(c3min,c3max,1)
                                tmp[1] = str(np.round(c3[0],2))
                                line= " ".join(tmp)
                                line= " " + line 

                        if tmp[0] == "4)":     # input image
                            if self.param4:
                                c4min, c4max = self.param4[0],self.param4[1]
                                c4 = np.random.default_rng().uniform(c4min,c4max,1)
                                tmp[1] = str(np.round(c4[0],2))
                                line= " ".join(tmp)
                                line= " " + line 


                        if tmp[0] == "5)":     # input image
                            if self.param5:
                                c5min, c5max = self.param5[0],self.param5[1]
                                c5 = np.random.default_rng().uniform(c5min,c5max,1)
                                tmp[1] = str(np.round(c5[0],2))
                                line= " ".join(tmp)
                                line= " " + line 

                               
                        if tmp[0] == "6)":     # input image
                            if self.param6:
                                c6min, c6max = self.param6[0],self.param6[1]
                                c6 = np.random.default_rng().uniform(c6min,c6max,1)
                                tmp[1] = str(np.round(c6[0],2))
                                line= " ".join(tmp)
                                line= " " + line 


                        if tmp[0] == "7)":     # input image
                            if self.param7:
                                c7min, c7max = self.param7[0],self.param7[1]
                                c7 = np.random.default_rng().uniform(c7min,c7max,1)
                                tmp[1] = str(np.round(c7[0],2))
                                line= " ".join(tmp)
                                line= " " + line 

                        if tmp[0] == "8)":     # input image
                            if self.param8:
                                c8min, c8max = self.param8[0],self.param8[1]
                                c8 = np.random.default_rng().uniform(c8min,c8max,1)
                                tmp[1] = str(np.round(c8[0],2))
                                line= " ".join(tmp)
                                line= " " + line 

                        if tmp[0] == "9)":     # input image
                            if self.param9:
                                c9min, c9max = self.param9[0],self.param9[1]
                                c9 = np.random.default_rng().uniform(c9min,c9max,1)
                                tmp[1] = str(np.round(c9[0],2))
                                line= " ".join(tmp)
                                line= " " + line 

                        if tmp[0] == "10)":     # input image
                            if self.param10:
                                c10min, c10max = self.param10[0],self.param10[1]
                                c10 = np.random.default_rng().uniform(c10min,c10max,1)
                                tmp[1] = str(np.round(c10[0],2))
                                line= " ".join(tmp)

                else:

                    if (tmp[0] == "3)") and (flagsky3  == False):     # input image
                        if self.param3:
                            c3min, c3max = self.param3[0],self.param3[1]
                            c3 = np.random.default_rng().uniform(c3min,c3max,1)
                            tmp[1] = str(np.round(c3[0],2))
                            line= " ".join(tmp)
                            line= " " + line 

                    if tmp[0] == "4)":     # input image
                        if self.param4:
                            c4min, c4max = self.param4[0],self.param4[1]
                            c4 = np.random.default_rng().uniform(c4min,c4max,1)
                            tmp[1] = str(np.round(c4[0],2))
                            line= " ".join(tmp)
                            line= " " + line 


                    if tmp[0] == "5)":     # input image
                        if self.param5:
                            c5min, c5max = self.param5[0],self.param5[1]
                            c5 = np.random.default_rng().uniform(c5min,c5max,1)
                            tmp[1] = str(np.round(c5[0],2))
                            line= " ".join(tmp)
                            line= " " + line 

                           
                    if tmp[0] == "6)":     # input image
                        if self.param6:
                            c6min, c6max = self.param6[0],self.param6[1]
                            c6 = np.random.default_rng().uniform(c6min,c6max,1)
                            tmp[1] = str(np.round(c6[0],2))
                            line= " ".join(tmp)
                            line= " " + line 


                    if tmp[0] == "7)":     # input image
                        if self.param7:
                            c7min, c7max = self.param7[0],self.param7[1]
                            c7 = np.random.default_rng().uniform(c7min,c7max,1)
                            tmp[1] = str(np.round(c7[0],2))
                            line= " ".join(tmp)
                            line= " " + line 

                    if tmp[0] == "8)":     # input image
                        if self.param8:
                            c8min, c8max = self.param8[0],self.param8[1]
                            c8 = np.random.default_rng().uniform(c8min,c8max,1)
                            tmp[1] = str(np.round(c8[0],2))
                            line= " ".join(tmp)
                            line= " " + line 

                    if tmp[0] == "9)":     # input image
                        if self.param9:
                            c9min, c9max = self.param9[0],self.param9[1]
                            c9 = np.random.default_rng().uniform(c9min,c9max,1)
                            tmp[1] = str(np.round(c9[0],2))
                            line= " ".join(tmp)
                            line= " " + line 

                    if tmp[0] == "10)":     # input image
                        if self.param10:
                            c10min, c10max = self.param10[0],self.param10[1]
                            c10 = np.random.default_rng().uniform(c10min,c10max,1)
                            tmp[1] = str(np.round(c10[0],2))
                            line= " ".join(tmp)




                lineout = line + "\n"
                fileout.write(lineout)

                if (tmp[0] == "0)") and (tmp[1] == "sky") :     # avoids to write in sky component
                    flagsky3 = True # sky component must be at the end of the file 
                                    # otherwise it will fail

                index += 1
            fileout.close()


#############################################################

    def MakeBash(self):

        #creating bash script

        listfiles = glob.glob("*.gal")

        bashout = open("rungalfit.sh", "w")

        lineout="#!/bin/bash \n" 
        bashout.write(lineout)

        for gfile in listfiles:

            lineout="galfit " + gfile  + "\n" 
            bashout.write(lineout)


        bashout.close()

        p = Path('rungalfit.sh')
        p.chmod(0o755)




#############################################################

    def ReadFile(self,inputf):

        File = open(inputf,"r")

        # All lines including the blank ones
        lines = (line.rstrip() for line in File)
        #lines = (line.split('#', 1)[0] for line in lines)  # remove comments
        #lines = (line.rstrip() for line in lines) # remove lines containing only comments
        lines = (line for line in lines if line)  # Non-blank lines

        lines = list(lines)

        return lines




#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  mainInitGal()





#!/usr/bin/env python3

import numpy as np
import sys
import os.path
from astropy.io import fits
from astropy.io.fits import CompImageHDU

import argparse

from matplotlib.path import Path




def maskDs9(MaskFile: str, RegFile: str, fill: bool, image: str, bor_flag: bool, borValue: int, skymean=None, skystd=None) -> None:

    if not os.path.exists(MaskFile):

        print ('%s: image filename does not exist!' %(MaskFile))
        print ('Creating a new image file ')

        hdu=fits.PrimaryHDU()

        if image:
            (nx, ny) = GetAxis(image)
        else:
            nx = input("enter numbers of pixels in X ") 
            ny = input("enter numbers of pixels in Y ") 

            nx = np.int64(nx)
            ny = np.int64(ny)

        Image = np.zeros([ny,nx])
        hdu.data = Image
        hdu.writeto(MaskFile,overwrite=True) 


    (ncol, nrow) = GetAxis(MaskFile)

    i = 0 #index of data 

    hdu = fits.open(MaskFile)

    Image = hdu[i].data



    if not os.path.exists(RegFile):
        print ('%s: reg filename does not exist!' %(sys.argv[2]))
        sys.exit(1)

    v0 = []
    v1 = []
    v2 = []
    v3 = []
    v4 = []
    v5 = []

    tupVerts = []
    Pol = []


    f1 = open(RegFile,'r')

    lines = f1.readlines()

    f1.close()

    flag = False
    flagpoly = False
    #reading reg file
    for line in lines:

        line = line.split("#")
        line = line[0]

        b1 = line.split("(")
        p = line.split(",")

        x1 = p[0]
        if b1[0] == "ellipse":

            x0 = "ellipse"
            x2 = x1[8:]
            flag = True

        if b1[0] == "box":

            x0 = "box"
            x2 = x1[4:]
            flag = True


        if b1[0] == "polygon":

            polv = line.split(")")
            pol, ver = polv[0].split("(")


            points = ver.split(",")

            N = len(points)

            i=0

            x = np.array([])
            y = np.array([])

            x = x.astype(float)
            y = y.astype(float)

            verts = []

            #make tuple for vertices
            while i < N:

                verts = verts + [(round(float(points[i])-1),round(float(points[i+1])-1)) ]
                i = i + 2

            flagpoly = True

        if (flag == True):

            x3 = p[4]
            x4 = x3[:-2]

            v0.append(x0)
            v1.append(float(x2) - 1)
            v2.append(float(p[1]) - 1)
            v3.append(float(p[2]))
            v4.append(float(p[3]))
            v5.append(float(x4))

            flag = False

        if (flagpoly  == True):

            Pol.append(pol) 
            tupVerts.append(verts)
            flagpoly = False

    obj  = np.array(v0)
    xpos = np.array(v1) 
    ypos = np.array(v2)
    rx = np.array(v3)
    ry = np.array(v4)
    angle = np.array(v5)


    #avoids ds9 regions with Area = 0
    maskrx = rx < 1
    if maskrx.any():
        rx[maskrx] = 1

    maskry = ry < 1
    if maskry.any():
        ry[maskry] = 1
    #



    Pol = np.array(Pol)

    for idx, item in enumerate(obj):

        #converting ellipse from DS9 ell to kron ellipse

        if obj[idx] == "ellipse":

            Image = MakeEllip(Image, fill, xpos[idx], ypos[idx], rx[idx], 
                                ry[idx], angle[idx], ncol, nrow, skymean=skymean, skystd=skystd)


        if obj[idx] == "box":


            Image = MakeBox(Image, fill, xpos[idx], ypos[idx], rx[idx], 
                            ry[idx], angle[idx], ncol, nrow, skymean=skymean, skystd=skystd)



    for idx, item in enumerate(Pol):

        #converting ellipse from DS9 ell to kron ellipse

        if Pol[idx] == "polygon":

            Image = MakePolygon(Image, fill, tupVerts[idx], ncol, nrow, skymean=skymean, skystd=skystd)



    if image:

        hduim=fits.open(image)
        dataImage = hduim[0].data

        #masking the border in case:
        bor_val = 100
        if bor_flag:
            #print("masking the border")
            bor_mask = dataImage == borValue 

            if bor_mask.any():
                Image[bor_mask] = bor_val 
 
        hduim.close()


    #writing mask file

    hdu.data=Image
    hdu.writeto(MaskFile,overwrite=True) 
    hdu.close()


def MakeEllip(Image, fill, xpos, ypos, rx, ry, angle, ncol, nrow, skymean=None, skystd=None):
    "Make an ellipse in an image"

    xx, yy, Rkron, theta, e = Ds9ell2Kronell(xpos,ypos,rx,ry,angle)
    (xmin, xmax, ymin, ymax) = GetSize(xx, yy, Rkron, theta, e, ncol, nrow)
    Image = MakeKron(Image, fill, xx, yy, Rkron, theta, e, xmin, xmax, 
                    ymin, ymax, ncol, nrow, skymean=skymean, skystd=skystd)

    return Image

def MakePolygon(Image, fill, tupVerts, ncol, nrow, skymean=None, skystd=None):
    "Make a polygon in an image"




    x, y = np.meshgrid(np.arange(ncol), np.arange(nrow)) # make a canvas with coordinates
    x, y = x.flatten(), y.flatten()
    points = np.vstack((x,y)).T 
    p = Path(tupVerts) # make a polygon

    grid = p.contains_points(points)
    mask = grid.reshape(nrow, ncol) # now you have a mask with points inside a polygon


    if skymean:

        sky = np.random.normal(skymean, skystd, (nrow, ncol))
        Image[mask] = sky[mask]  

    else:

        Image[mask] = fill





    return Image





def MakeBox(Image,fill,xpos,ypos,rx,ry,angle,ncol,nrow, skymean=None, skystd=None):
    "Make a box in an image"


    anglerad = angle*np.pi/180
    beta = np.pi/2 - anglerad

    lx = (rx/2)*np.cos(anglerad) - (ry/2)*np.cos(beta)
    lx2 = (rx/2)*np.cos(anglerad) + (ry/2)*np.cos(beta)
    ly = (rx/2)*np.sin(anglerad) + (ry/2)*np.sin(beta)
    ly2 = (rx/2)*np.sin(anglerad) - (ry/2)*np.sin(beta)


    v1x = round(xpos - lx)  
    v1y = round(ypos - ly) 


    v2x = round(xpos - lx2) 
    v2y = round(ypos - ly2) 


    v3x = round(xpos + lx)  
    v3y = round(ypos + ly) 


    v4x = round(xpos + lx2)
    v4y = round(ypos + ly2)



    Verts = [(v1x,v1y),(v2x,v2y),(v3x,v3y),(v4x,v4y)]



    x, y = np.meshgrid(np.arange(ncol), np.arange(nrow)) # make a canvas with coordinates
    x, y = x.flatten(), y.flatten()
    points = np.vstack((x,y)).T 

    p = Path(Verts) # make a polygon

    grid = p.contains_points(points)

    mask = grid.reshape(nrow, ncol) # now you have a mask with points inside a polygon


    if skymean:

        sky = np.random.normal(skymean, skystd, (nrow, ncol))
        Image[mask] = sky[mask]  

    else:

        Image[mask] = fill



    return Image


def GetAxis(Image):
    # k Check
    "Get number of rows and columns from the image"

    i = 0 #index indicated where the data is located

    if(checkCompHDU(Image)):
        i=1


    hdu = fits.open(Image)


    ncol = hdu[i].header["NAXIS1"]
    nrow = hdu[i].header["NAXIS2"]



    hdu.close()

    return ncol, nrow


def checkCompHDU(file):
    """ check if fits file is a CompImageHDU"""
    flag = False
    hdul = fits.open(file) 

    for i, hdu in enumerate(hdul):

        if isinstance(hdu, CompImageHDU):
            flag = True

    hdul.close()

    return flag 



def Ds9ell2Kronell(xpos,ypos,rx,ry,angle):


    if rx >= ry:

        q = ry/rx
        e = 1 - q
        Rkron = rx
        theta = angle
        xx = xpos
        yy = ypos
    else:
        q = rx/ry
        e = 1 - q
        Rkron = ry
        theta = angle + 90
        xx = xpos
        yy = ypos
 
     
    return xx, yy, Rkron, theta, e 



def MakeKron(imagemat, idn, x, y, R, theta, ell, xmin, xmax, ymin, ymax, ncol, nrow, skymean=None, skystd=None):
    "This subroutine create a Kron ellipse within a box defined by: xmin, xmax, ymin, ymax"

    # Check

    xmin = int(xmin)
    xmax = int(xmax)
    ymin = int(ymin)
    ymax = int(ymax)

    q = (1 - ell)
    bim = q * R

    theta = theta * np.pi / 180  # Rads!!!

    ypos, xpos = np.mgrid[ymin - 1 : ymax + 1, xmin - 1: xmax +1]

    dx = xpos - x
    dy = ypos - y

    landa = np.arctan2(dy, dx)

    mask = landa < 0
    if mask.any():
        landa[mask] = landa[mask] + 2*np.pi

    landa = landa - theta

    angle = np.arctan2(np.sin(landa) / bim, np.cos(landa) / R)

    xell = x + R * np.cos(angle) * np.cos(theta) - bim * \
        np.sin(angle) * np.sin(theta)
    yell = y + R * np.cos(angle) * np.sin(theta) + bim * \
        np.sin(angle) * np.cos(theta)

    dell = np.sqrt((xell - x)**2 + (yell - y)**2)
    dist = np.sqrt(dx**2 + dy**2)

    mask = dist <= dell


    if skymean:

        sky = np.random.normal(skymean, skystd, (nrow, ncol))
        imagemat[ypos[mask], xpos[mask]] = sky[ypos[mask], xpos[mask]]

    else:

        imagemat[ypos[mask], xpos[mask]] = idn

    return imagemat


def GetSize(x, y, R, theta, ell, ncol, nrow):
    "this subroutine get the maximun"
    "and minimim pixels for Kron and sky ellipse"
    # k Check
    q = (1 - ell)
    bim = q * R

    theta = theta * (np.pi / 180)  # rads!!

    # getting size

    constx =  np.sqrt((R**2)*(np.cos(theta))**2 + (bim**2)*(np.sin(theta))**2)
    consty =  np.sqrt((R**2)*(np.sin(theta))**2 + (bim**2)*(np.cos(theta))**2)



    xmin = x - constx                       
    xmax = x + constx
    ymin = y - consty
    ymax = y + consty
                    

    mask = xmin < 1
    if mask.any():
        xmin = 1

    mask = xmax > ncol
    if mask.any():
        xmax = ncol - 1

    mask = ymin < 1
    if mask.any():
        ymin = 1

    mask = ymax > nrow
    if mask.any():
        ymax = nrow - 1

    return (round(xmin), round(xmax), round(ymin), round(ymax))



  
#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  mainMaskDs9()





#! /usr/bin/env python3 


import numpy as np

import re


def log2csv(num:int, fileout: str):


    file = "fit.log"

    #fileout="fitlog.csv"

    fitlogF = open(file,"r")

    f_out = open(fileout, "w")

    # All lines including the blank ones
    lines = (line.rstrip() for line in fitlogF)
    lines = (line.split('#', 1)[0] for line in lines)  # remove comments
    # remove lines containing only comments
    lines = (line.rstrip() for line in lines)
    lines = (line for line in lines if line)  # Non-blank lines

    lines = list(lines)
    index = 0

    linnp = np.array(lines)
    idxfits = np.where(linnp == "-----------------------------------------------------------------------------")



    if num:
        #use user number 
        index = idxfits[0][2*num-2]
        cnt=num*2-1
    else:
        #use last one by default
        index = idxfits[0][-2]
        cnt=int(len(idxfits[0])-1)



    line = lines[index]
    (tmp) = line.split()

    if tmp[0] == "-----------------------------------------------------------------------------":

        index += 1
        lineout = lines[index]+"\n"
        f_out.write(lineout)

        index += 1
        lineout = lines[index]+"\n"
        f_out.write(lineout)

        index += 1
        lineout = lines[index]+"\n"
        f_out.write(lineout)


        index += 1
        lineout = lines[index]+"\n"
        f_out.write(lineout)

        #put chi and sky at the beggining

        index2=idxfits[0][cnt] - 4


        line= lines[index2]
        line=re.sub('[^A-Za-z0-9.+.-]+', ' ', line)
        (tmp) = line.split()
        lineout1 = "sky = "+tmp[3]+", "+tmp[4]+", "+tmp[5]

        index2 += 1
        line= lines[index2]
        line=re.sub('[^A-Za-z0-9.+.-]+', ' ', line)
        (tmp) = line.split()


        lineout2 = " skyerr = "+tmp[0]+", "+tmp[1]+", "+tmp[2]

        lineout = lineout1 + "," + lineout2+"\n"

        f_out.write(lineout)


        index2 += 1



        line= lines[index2]
        line=re.sub('[^A-Za-z0-9.+]+', ' ', line)
        (tmp) = line.split()

        lineout1 = "Chi^2 = " + tmp[2] + " ndof = " + tmp[4]

        index2 += 1
        line = lines[index2]
        (tmp) = line.split()

        lineout2 = " ".join(tmp) 

        lineout = lineout1 + " " + lineout2+"\n"

        f_out.write(lineout)


        cnt+=2

        lineout="model parameters, error parameters\n" 
        f_out.write(lineout)
 

        lineout="------------------------------------------------------------\n" 
        f_out.write(lineout)
 
        
        #end of header

        index += 1
        line = lines[index]
        line=re.sub('[^A-Za-z0-9.+.-]+', ' ', line)
        (tmp) = line.split()

        while tmp[0] != "sky":
 
                
            lineout1 = ",".join(tmp) 

            index += 1
            line= lines[index]
            line=re.sub('[^A-Za-z0-9.+.-]+', ' ', line)
            (tmp) = line.split()


            lineout2 = ",".join(tmp) 

            lineout = lineout1 + "," + lineout2+"\n"

            f_out.write(lineout)


            index += 1
            line= lines[index]
            line=re.sub('[^A-Za-z0-9.+.-]+', ' ', line)
            (tmp) = line.split()



    fitlogF.close()
    f_out.close()







import pytest
import os


import subprocess as sp

from galfitools.galout.getRads import getBreak
from galfitools.galout.getRads import getBreak2
from galfitools.galout.getRads import getFWHM
from galfitools.galout.getRads import getKappa
from galfitools.galout.getRads import getReComp
from galfitools.galout.getRads import getSlope
from galfitools.galout.getRads import getBulgeRad
from galfitools.galout.getMissingLight import getMissLight
from galfitools.galout.getN import getN
from galfitools.galout.showcube import displayCube

from galfitools.galout.PhotDs9 import photDs9 




def test_getBreak():



    galfitFile = "galfit.galout" 

    path="tests/"
    galfitFile=path+galfitFile
 


    dis = 10 

    inicomp = 2 

    quick = False 
    random = False 

    num_comp =  1 

    angle = None 


    ranx = None 
    plot = False 


    result = 248.62

    tol = 1e-2

    rbreak, N, theta = getBreak(galfitFile, dis, inicomp, quick, random, num_comp, angle, plot, ranx)
    

    diffrbreak = abs(rbreak -result)


    assert diffrbreak < tol



    return None




def test_getBreak2():

    galfitFile = "galfit.galout" 
    path="tests/"
    galfitFile=path+galfitFile
 
    dis = 10 
    angle = None 
    num_comp = 1 
    plot = False 
    ranx = None 


    rbreak, N, theta =  getBreak2(galfitFile, dis, angle, num_comp, plot, ranx)

    result = 42.7 

    tol = 1e-2


    diffrbreak = abs(rbreak -result)


    assert diffrbreak < tol





    return None

def test_getFWHM():

    galfitFile = "galfit.galout" 
    path="tests/"
    galfitFile=path+galfitFile
 
    dis = 10 
    angle = None 

   
    num_comp = 1 


    fwhm, N, theta = getFWHM(galfitFile, dis, angle, num_comp)

    result = 8.81 

    tol = 1e-2


    difffwhm = abs(fwhm - result)


    assert difffwhm < tol



    return None


def test_getKappa():

    galfitFile = "galfit.galout" 
    dis = 10 
    path="tests/"
    galfitFile=path+galfitFile
 

    inicomp = 2 
    quick = False 
    random = False 
    num_comp =  1 
    angle = None 
    ranx = None 
    plot = False 





    rkappa, N, theta = getKappa(galfitFile, dis, inicomp, quick, random, angle, num_comp, plot, ranx) 

    result = 2.62 
    tol = 1e-2


    diffrkappa = abs(rkappa - result)


    assert diffrkappa < tol




    return None


def test_getReComp():


    galfitFile = "galfit.galout" 
    path="tests/"
    galfitFile=path+galfitFile
 
    dis = 10 
    eff = .5  
    num_comp = 1 
    angle = None 



    EffRad, totmag, meanme, me, N, theta = getReComp(galfitFile, dis, eff, angle, num_comp)


    result = 97.82 
    tol = 1e-2


    diffEffRad = abs(EffRad - result)


    assert diffEffRad < tol


    return None



def test_getSlope():


    galfitFile = "galfit.galout" 
    dis = 10
    path="tests/"
    galfitFile=path+galfitFile
 
    slope = 0.5 

    num_comp = 1 

    angle = None 

    ranx = None 
    plot = False 




    rgam, N, theta = getSlope(galfitFile, dis, slope, angle, num_comp, plot, ranx)

    result = 2.62 
    tol = 1e-2


    diffrgam = abs(rgam - result)


    assert diffrgam < tol





    return None



def test_getBulgeRad():

    galfitFile1 = "galfit.1ser"
    galfitFile2 = "galfit.3ser"
    path="tests/"
    galfitFile1=path+galfitFile1
    galfitFile2=path+galfitFile2
 
    dis = 10 

    num_comp =  1 

    angle = None 

    ranx = None 
    plot = False 


    rbulge, N1, N2, theta = getBulgeRad(galfitFile1, galfitFile2, dis, num_comp, angle, plot, ranx)

    result = 3.86
    tol = 1e-2


    diffrbulge= abs(rbulge - result)


    assert diffrbulge < tol





    return None

def test_getMissLight():

    galfitFile1 = "galfit.1ser"
    galfitFile2 = "galfit.3ser"
    path="tests/"
    galfitFile1=path+galfitFile1
    galfitFile2=path+galfitFile2
 

    dis = 10 

    num_comp = 1 

    rad = 3.86  



    magmiss, N1, N2 = getMissLight(galfitFile1, galfitFile2, dis, num_comp, rad)

    result = 18.66 
    tol = 1e-2


    diffmiss = abs(magmiss - result)


    assert diffmiss < tol




def test_getN():

    galfitFile = "galfit.galout" 
    dis = 10 
    path="tests/"
    galfitFile=path+galfitFile
 
   
    num_comp =  1

    frac = .2 #irrelevant variable, it is not used anymore but it was kept 

    angle = None

    plot = False 


    sersic, meanser, stdser, totmag, N, theta = getN(galfitFile, dis, frac, angle, num_comp, plot)




    result1 = 3.48 
    result2 = 2.66 

    tol = 1e-2


    diffser = abs(sersic - result1)

    assert diffser < tol



    diffser2 = abs(meanser - result2)

    assert diffser2 < tol



    return None


def test_displayCube():

    cubeimage =  "A1656-showcube.fits"
    namecube = "cube.png" 
    dpival = 100 
    brightness = 0
    contrast = 1 
    cmap = "viridis" 
    scale = 1 
    noplot = False 

    path="tests/"
    cubeimage=path+cubeimage
    namecube=path+namecube
 
    displayCube(cubeimage, namecube, dpival, brightness, contrast, cmap, scale, noplot)

    assert os.path.isfile(namecube)

    if os.path.isfile(namecube):
        os.remove(namecube)




    return None


def test_photDs9():

    ImageFile =   "A671.gtMakeMask.maskds9.masksky.fits"
    RegFile = "maskds9.reg" 
    path = "tests/"
    ImageFile = path+ImageFile
    RegFile = path+RegFile
 
    maskfile = "none" 

    zeropoint = 25 
    sky = 0 



    mag, exptime = photDs9(ImageFile, RegFile, maskfile ,zeropoint, sky)


    result =  11.71
    tol = 1e-2


    diffmag = abs(mag - result)


    assert diffmag < tol




    return None








#!/usr/bin/env python3

import numpy as np
import sys
import os
import stat
import subprocess as sp
import os.path
from astropy.io import fits
import scipy
import scipy.special
import matplotlib.pyplot as plt

import argparse


from galfitools.galin.MaskDs9 import GetAxis 

#################################################################
#################################################################

def skyRem(imageFile, maskFile, mean, sig, nsig, borValue, bor_flag=False):

    bor_val = 100

    if not os.path.exists(imageFile):
        print ('{} image filename does not exist!'.format(imageFile))
        sys.exit()


    (ncol, nrow) = GetAxis(imageFile)



    if not os.path.exists(maskFile):
        print ('{} mask image does not exist, creating one ... '.format(maskFile)) 
    else:
        print ('overwriting {} mask image '.format(maskFile)) 

    MakeImage(maskFile, ncol, nrow)




    i = 0 #index of data 
    # original file
    hdu = fits.open(imageFile)
    dataImage = hdu[i].data


    # output file
    hdu2 = fits.open(maskFile)
    maskImage = hdu2[0].data


    topsky = mean + sig*nsig


    mask = dataImage >= topsky 



    if mask.any():
        maskImage[mask] = dataImage[mask] - topsky 


    #masking the border in case:

    if bor_flag:
        print("masking the border")
        bor_mask = dataImage == borValue 

        if bor_mask.any():
            maskImage[bor_mask] = bor_val 
 


    #writing mask file:

    hdu2[0].data = maskImage

    hdu2.writeto(maskFile, overwrite=True)


    hdu.close()
    hdu2.close()






def MakeImage(newfits, sizex, sizey):
    "create a new blank Image"

    if os.path.isfile(newfits):
        print("{} deleted; a new one is created \n".format(newfits))

        runcmd = "rm {}".format(newfits)
        errrm = sp.run([runcmd], shell=True, stdout=sp.PIPE,
                        stderr=sp.PIPE, universal_newlines=True)


    hdu = fits.PrimaryHDU()
    hdu.data = np.zeros((sizey, sizex),dtype=np.float64)
    hdu.writeto(newfits, overwrite=True)

    return True




  
#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
  mainMaskSky()







import pytest
import os


import subprocess as sp



from galfitools.sim.MakeSim import makeSim


def test_makeSim():

    image = "M81-BD.fits" 
    path="tests/"
    image = path+image
    GAIN = 6.5 

    skymean = 0 
    skystd = 1  

    newimage = "M81Sim.fits" 
    newimage= path+newimage



    makeSim(image, GAIN, skymean, skystd, newimage)


    assert os.path.isfile("poissonimg.fits")

    if os.path.isfile("poissonimg.fits"):
        os.remove("poissonimg.fits")

    assert os.path.isfile("skynoise.fits")

    if os.path.isfile("skynoise.fits"):
        os.remove("skynoise.fits")




    assert os.path.isfile(newimage)

    if os.path.isfile(newimage):
        os.remove(newimage)



    return None



#check if galfit is installed
def test_galfit():

    runcmd="galfit -help"
    err = sp.run([runcmd],shell=True,stdout=sp.PIPE,stderr=sp.PIPE,universal_newlines=True)  

    assert err.returncode == 0, "is GALFIT installed?"








import pytest
import os


import subprocess as sp



from galfitools.galin.getStar import getStar
from galfitools.galin.initgal import InitGal
from galfitools.galin.MaskDs9 import maskDs9

from galfitools.galin.MakeMask import makeMask
from galfitools.galin.MaskSky import skyRem
from galfitools.galin.xy2fits import xy2fits



def test_getStar():


    image = "A671.gtMakeMask.maskds9.masksky.fits"
    regfile = "ds9.getStar.reg"

    path="tests/"

    image = path+image
    regfile = path+regfile

    imsize = 70 
    center = False 

    sky = 1153 
    imout = "testgetStar.fits" 

    imout = path+imout

    sigma = None 
    sigout = None 

    getStar(image, regfile, imsize, center, sky, imout, sigma, sigout)


    assert os.path.isfile(imout)

    if os.path.isfile(imout):
        os.remove(imout)



    return None


def test_InitGal():

    GalfitFile = "galfit.initgal"


    path="tests/"
    GalfitFile=path+GalfitFile

    number = 1 
    param3 = [1,50] 
    param4 = None 
    param5 = None 
    param6 = None 
    param7 = None 
    param8 = None 
    param9 = None 
    param10 = None 


    numcomp = 1


    InitGal(GalfitFile, number, param3, param4, param5, param6, param7, param8, param9, param10, numcomp)


    fileout="galfit-1.gal"

    fileout = path+fileout

    assert os.path.isfile(fileout)

    if os.path.isfile(fileout):
        os.remove(fileout)


    namefile = "rungalfit.sh"
    #namefile = path+namefile

    assert os.path.isfile(namefile)
    if os.path.isfile(namefile):
        os.remove(namefile)






    return None



def test_maskDs9():


    MaskFile = "tempmask.fits" 
    RegFile = "maskds9.reg" 
    fill = 100 
    image = "A671.gtMakeMask.maskds9.masksky.fits"

    
    path="tests/"
    MaskFile = path+MaskFile
    RegFile = path + RegFile
    image = path + image 

    bor_flag = False 
    borValue = 100 


    maskDs9(MaskFile, RegFile, fill, image, bor_flag, borValue) 

    assert os.path.isfile(MaskFile)

    if os.path.isfile(MaskFile):
        os.remove(MaskFile)


    return None


def test_makeMask():

    sexfile = "cold.gtMakeMask"
    image =  "A671.gtMakeMask.maskds9.masksky.fits"
    maskfile = "tempmakemask.fits" 
    scale = 1 
    satfileout = "ds9sat.reg" 

    path="tests/"
    sexfile = path+sexfile
    image=path+image
    maskfile = path+maskfile


    makeMask(sexfile, image, maskfile, scale, satfileout)

    assert os.path.isfile(maskfile)

    if os.path.isfile(maskfile):
        os.remove(maskfile)

    namefile = "sexsort.cat"
    #namefile = path+namefile

    assert os.path.isfile(namefile)
    if os.path.isfile(namefile):
        os.remove(namefile)



    assert os.path.isfile(satfileout)
    if os.path.isfile(satfileout):
        os.remove(satfileout)



    return None



def test_skyRem():


    image =  "A671.gtMakeMask.maskds9.masksky.fits"
    mask = "tempmasksky.fits" 

    path="tests/"
    image  = path+image 
    mask = path + mask

    sky_mean = 1150
    sky_sig = 14
    nsig = 1 

    bor_flag = False 
    borValue = 100 


    skyRem(image, mask, sky_mean, sky_sig, nsig, borValue, bor_flag)

    assert os.path.isfile(mask)

    if os.path.isfile(mask):
        os.remove(mask)



    return None

def test_xy2fits():


    ImageFile=  "A671.gtMakeMask.maskds9.masksky.fits"

    AsciiFile= "maskscii.txt" 

    path="tests/"
    ImageFile=path+ImageFile
    AsciiFile=path+AsciiFile

    Value = 100 

    xy2fits().MakeFits(ImageFile, AsciiFile, Value)

    maskfile = "maskscii.fits"
    maskfile=path+maskfile

    assert os.path.isfile(maskfile)

    if os.path.isfile(maskfile):
        os.remove(maskfile)



    return None












#!/usr/bin/env python3

import numpy as np
import sys
import os
import stat
import subprocess as sp
import os.path
from astropy.io import fits
import scipy
import scipy.special
import matplotlib.pyplot as plt
from scipy.special import gammaincinv

import mgefit
from mgefit.find_galaxy import find_galaxy
from mgefit.sectors_photometry import sectors_photometry
from mgefit.sectors_photometry_twist import sectors_photometry_twist


from mgefit.mge_fit_sectors import mge_fit_sectors
from mgefit.mge_fit_sectors_twist import mge_fit_sectors_twist

from mgefit.mge_fit_sectors_regularized import mge_fit_sectors_regularized 

import argparse


#from galfitools.galin.galfit import Galfit, conver2Sersic, SelectGal, numComps, GetRadAng
from galfitools.galin.galfit import Galfit
from galfitools.galin.MaskDs9 import GetAxis 

def  mge2gal(galfitFile, regfile, center, psf, twist, gauss, freeser, 
                freesky, numgauss, xypos=None, ellip = None, posang = None) -> str:


    # reading options from galfit header
    galfit = Galfit(galfitFile)
    head = galfit.ReadHead()
    galsky = galfit.ReadSky()

    image = head.inputimage
    imageout = head.outimage
    magzpt = head.mgzpt
    maskfile = head.maskimage

    sky = galsky.sky

    scale = head.scale
    psfile = head.psfimage

    sigfile = head.sigimage

    convbox =  head.convx
    convboxy = head.convy

    consfile = head.constraints
    xlo = head.xmin
    ylo = head.ymin
    xhi = head.xmax
    yhi = head.ymax
 


    #################

    initgauss = 12
    regu = None #regu option now available


    mgeoutfile="mgegas.txt"



    ######Updating variables for empty files ###########

    if (maskfile == "None") or (maskfile == "none"):
        maskfile = None

    if (sigfile == "None") or  (sigfile == "none"):
        sigfile = None

    if (psfile == "None") or (psfile == "none"):
        psfile = None


    if regu:
        try:
            from mgefit.mge_fit_sectors_twist_regularized import mge_fit_sectors_twist_regularized 
        except: 
            print("mge_fit_sectors_twist_regularized is not installed. Regu desactivated")
            regu = False

    ##########################################
    ############################################


    if regu:
        print('regularization mode activated')


    root_ext = os.path.splitext(image)
    timg= root_ext[0]


    namepng=timg+".png"

    sectorspng="sectors.png"

    magzpt=float(magzpt)



    fit=1
    serfit=0
    skyfit=0

    Z=0


    ###################################################
    #  Create GALFIT input file header to compute sky #
    ###################################################

    #    image = "ngc4342.fits"

    root_ext = os.path.splitext(imageout)
    TNAM= root_ext[0]




    exptime = GetExpTime(image)



    ######################
    #    Mask file
    #################
    if maskfile:

        errmsg="file {} does not exist".format(maskfile)
        assert os.path.isfile(maskfile), errmsg
        hdu = fits.open(maskfile)
        mask = hdu[0].data
        maskb=np.array(mask,dtype=bool)
        maskbt=maskb.T
        hdu.close()

    else:
        mask=np.array([])

    ######################
    #    sky=back
    ######################

    hdu = fits.open(image)
    img = hdu[0].data

    img=img.astype(float)

    minlevel = 0  

    plt.clf()

    (ncol, nrow) = GetAxis(image)

    eps = 0 
    theta = 0

    if regfile:
        if os.path.isfile(regfile):

            obj, xpos, ypos, rx, ry, angle = GetInfoEllip(regfile)


            xx, yy, Rkron, theta, eps = Ds9ell2Kronellv2(xpos,ypos,rx,ry,angle)
            if center:
                print('center of ds9 ellipse region will be used')
                xpeak, ypeak = xpos, ypos
            else:        
                (xmin, xmax, ymin, ymax) = GetSize(xx, yy, Rkron, theta, eps, ncol, nrow)
                xpeak, ypeak = GetPmax(img, mask, xmin, xmax, ymin, ymax)


    #ignore the previous data of the object if any of the next options flags are activated 
    if xypos:
        xpeak = xypos[0] - 1
        ypeak = xypos[1] - 1

    if ellip:
        eps = ellip

    if posang:
        theta = posang 

    print("galaxy found at ", xpeak + 1, ypeak + 1)
    print("Ellipticity, Angle = ", eps, theta)

    print("Sky = ", sky)

    img = img - sky   # subtract sky

    # I have to switch x and y coordinates, don't ask me why
    xtemp=xpeak
    xpeak=ypeak
    ypeak=xtemp

    plt.clf()

    if twist:
        print("twist mode mge fit sectors is used")
                        # Perform galaxy photometry
        if maskfile:
            s = sectors_photometry_twist(img, theta, xpeak, ypeak, 
                                         minlevel=minlevel, badpixels=maskb ,plot=1)
        else:
            s = sectors_photometry_twist(img, theta, xpeak, ypeak, minlevel=minlevel, 
                                         plot=1)

        plt.savefig(sectorspng)

        plt.pause(1)  # Allow plot to appear on the screen

        plt.clf()

        tolpsf = 0.001

        if np.abs(psf) > tolpsf:
            if regu:
                m = mge_fit_sectors_twist_regularized(s.radius, s.angle, s.counts, eps, ngauss=initgauss,
                                            sigmapsf=psf, scale=scale, plot=1)
            else:
                m = mge_fit_sectors_twist(s.radius, s.angle, s.counts, eps, ngauss=initgauss,
                                            sigmapsf=psf, scale=scale, plot=1)
 
        else:
            print("No convolution")
            if regu:
                m = mge_fit_sectors_twist_regularized(s.radius, s.angle, s.counts, eps, ngauss=initgauss,
                                             scale=scale, plot=1)
            else:
                m = mge_fit_sectors_twist(s.radius, s.angle, s.counts, eps, ngauss=initgauss,
                                             scale=scale, plot=1)
 


        plt.pause(1)  # Allow plot to appear on the screen

        plt.savefig(namepng)


    else:
    #elif twist == False:

        if maskfile:
            s = sectors_photometry(img, eps, theta, xpeak, ypeak, minlevel=minlevel, badpixels=maskb ,plot=1)
        else:

            s = sectors_photometry(img, eps, theta, xpeak, ypeak, minlevel=minlevel, plot=1)

#            s = sectors_photometry(img, eps, theta, xpeak, ypeak, minlevel=minlevel, plot=1)

        plt.pause(1)  # Allow plot to appear on the screen

        plt.savefig(sectorspng)

    ###########################

        plt.clf()



        tolpsf = 0.001
        if np.abs(psf) > tolpsf:

            if regu:
 
                m = mge_fit_sectors_regularized(s.radius, s.angle, s.counts, eps,
                                    ngauss=initgauss, sigmapsf=psf,
                                    scale=scale, plot=1, bulge_disk=0, linear=0)
            else:
                m = mge_fit_sectors(s.radius, s.angle, s.counts, eps,
                                    ngauss=initgauss, sigmapsf=psf,
                                    scale=scale, plot=1, bulge_disk=0, linear=0)
 

        else:
            print("No convolution")

            if regu:
                m = mge_fit_sectors_regularized(s.radius, s.angle, s.counts, eps,
                                    ngauss=initgauss, scale=scale, plot=1, 
                                    bulge_disk=0, linear=0)
            else:
                m = mge_fit_sectors(s.radius, s.angle, s.counts, eps,
                                    ngauss=initgauss, scale=scale, plot=1, 
                                    bulge_disk=0, linear=0)
 

        plt.pause(1)  # Allow plot to appear on the screen

        plt.savefig(namepng)






    if twist:

        (counts, sigma, axisrat, pa) = m.sol

        theta2 = 270 - theta
        alpha1 = pa - theta2
        alphaf = alpha1 - 90

    else:
        #elif twist == False:
        (counts, sigma, axisrat) = m.sol
        #anglegass = 90 - theta
        anglegass =  theta


    ####################
    ### print GALFIT files
    #####################

    if gauss:
        parfile="mgeGALFIT.txt"
    else:
        parfile="mseGALFIT.txt"

    outname=TNAM

    rmsname = sigfile 
    if psfile:
        psfname = psfile
    else:
        psfname = "None"


    # switch back
    xtemp=xpeak
    xpeak=ypeak
    ypeak=xtemp



    T1 = "{}".format(image)
    T2 = outname + "-mge.fits"
    T3 = "{}".format(rmsname)


    # now this is read from galfit file

    #xlo=1
    #ylo=1
    #(xhi, yhi) = (ncol, nrow)



    fout1 = open(parfile, "w")
    fout2 = open(mgeoutfile, "w")


    outline2 = "# Mag Sig(pixels) FWHM(pixels) q angle \n"
    fout2.write(outline2)

    #sersic K for n = 0.5

    K = gammaincinv(1,0.5)


    if psfile:
        (ncol, nrow) = GetAxis(psfname)
        convbox = ncol + 1
        convboxy = nrow + 1



    PrintHeader(fout1, T1, T2, T3, psfname, 1, maskfile, consfile, xlo, xhi, ylo,
                yhi, convbox, convboxy, magzpt, scale, scale, "regular", 0, 0)


    if freeser:
        serfit = 1
    else:
        serfit = 0


    #removing the last components:
    totGauss = len(counts)

    if not(numgauss):
        numgauss = totGauss

    print("total number of gaussians by mge_fit_sectors: ", totGauss)

    if numgauss > totGauss:
        print("number of gaussians is greater than the ones fitted by mge_fit_sectors")
        print("total number of gaussians of mge_fit_sectors will be used instead")
        numgauss = totGauss

    print("total number of gaussians used by GALFIT: ", numgauss)



    index = 0

    while index < numgauss: 


        TotCounts = counts[index]
        SigPix =  sigma[index]
        qobs =  axisrat[index]

        TotCounts = float(TotCounts)
        SigPix = float(SigPix)
        qobs = float(qobs)

        if twist:
            anglegass = alphaf[index] #- 90
            anglegass = float(anglegass)


        if gauss:
            
            C0=TotCounts/(2*np.pi*qobs*SigPix**2)

            Ftot = 2*np.pi*SigPix**2*C0*qobs

            mgemag = magzpt + 0.1  + 2.5*np.log10(exptime)  - 2.5*np.log10(Ftot)

            FWHM = 2.35482*SigPix

            outline = "Mag: {:.2f}  Sig: {:.2f}  FWHM: {:.2f}  q: {:.2f} angle: {:.2f} \n".format(mgemag, SigPix, FWHM, qobs, anglegass)
            print(outline)

            outline2 = "{:.2f} {:.2f} {:.2f} {:.2f} {:.2f} \n".format(mgemag, SigPix, FWHM, qobs, anglegass)
            fout2.write(outline2)


            PrintGauss(fout1, index+1, xpeak + 1, ypeak + 1, mgemag, FWHM, qobs, anglegass, Z, fit)


        else:

            C0=TotCounts/(2*np.pi*qobs*SigPix**2)

            Ftot = 2*np.pi*SigPix**2*C0*qobs

            mgemag = magzpt + 0.1  + 2.5*np.log10(exptime)  - 2.5*np.log10(Ftot)
            
            FWHM = 2.35482*SigPix

            h  = np.sqrt(2) * SigPix
             
            Re = (K**(0.5))*h

            outline = "Mag: {:.2f}  Sig: {:.2f}  FWHM: {:.2f} Re: {:.2f}  q: {:.2f} angle: {:.2f} \n".format(mgemag, SigPix, FWHM, Re, qobs, anglegass)
            print(outline)

            outline2 = "{:.2f} {:.2f} {:.2f} {:.2f} {:.2f} {:.2f} \n".format(mgemag, SigPix, FWHM, Re,  qobs, anglegass)
            fout2.write(outline2)


            PrintSersic(fout1, index+1, xpeak + 1, ypeak + 1, mgemag, Re, 0.5, qobs, anglegass, Z, fit, serfit)



        index+=1


    if freesky:
        skyfit = 1
    else:
        skyfit = 0

    PrintSky(fout1, index+1, sky, Z, skyfit)
    fout1.close()
    fout2.close()

    makeConstraints(consfile, len(counts))

    print("Done. Gaussians are stored in {}, and {} for galfit format ".format(mgeoutfile,parfile))

    #returns output name
    return T2
####################################################
####################################################
#####################################################

def ReadMgePsf(psfile):


    counts,mag,sigpix,fwhm,qobs,angle = np.genfromtxt(psfile, skip_header=1, delimiter="", unpack=True)

    tot=counts.sum()
    psfs=sigpix

    normpsf=counts/tot

    return psfs,normpsf


def Sextractor(filimage,X,Y):

    sexfile="default.sex"
    CatSex="sim.cat"

    runcmd="sextractor -c {} {} ".format(sexfile,filimage)
    print(runcmd)

    err = sp.run([runcmd],shell=True,stdout=sp.PIPE,stderr=sp.PIPE,universal_newlines=True)  # Run GALFIT

    KronScale=1
###############  Read in sextractor sorted data ################
    if os.stat(CatSex).st_size != 0 :
        Num, RA, Dec, XPos, YPos, Mag, Kron, FluxRad, IsoArea, AIm, E, Theta, Background, Class, Flag = np.genfromtxt(
           CatSex, delimiter="", unpack=True)  # sorted
## checking if they are numpy arrays:


#        index =  Mag.argsort()
#        i=0
        dist=100

        for idx, item in enumerate(Num):


#        if(isinstance(Num,np.ndarray)):
            dx= XPos[idx] - X
            dy= YPos[idx] - Y
            dt= np.sqrt(dx**2 + dy**2)

            cflag=CheckFlag(4,Flag[idx])
            if cflag==False and dt < dist:
                dist = dt
                sindex=idx



        Num=Num[sindex]
        RA=RA[sindex]
        Dec=Dec[sindex]
        XPos=XPos[sindex]
        YPos=YPos[sindex]
        Mag=Mag[sindex]
        Kron=Kron[sindex]
        FluxRad=FluxRad[sindex]
        IsoArea=IsoArea[sindex]
        AIm=AIm[sindex]
        E=E[sindex]
        Theta=Theta[sindex]
        Background=Background[sindex]
        Class=Class[sindex]
        Flag=Flag[sindex]

##
        Angle = np.abs(Theta)
        AR = 1 - E
        RKron = KronScale * AIm * Kron
        Sky = Background


############

        XPos=round(XPos)
        YPos=round(YPos)

        XPos=int(XPos)
        YPos=int(YPos)

    else:
        print("Sextractor cat file not found ")

    print("Sextractor done")
    return E,Angle,XPos,YPos,Background






### GALFIT functions

def PrintHeader(hdl, A, B, C, D, E, F, G, xlo, xhi, ylo, yhi, convx, convy, J, platedx, platedy, O, P, S):
    "print GALFIT header in a file"

    # k Check
    # print to filehandle
    # the header for GALFIT

    lineZ = "==================================================================================================\n"
    lineX = "# IMAGE PARAMETERS \n"
    lineA = "A) {}    # Input Data image (FITS file)                            \n".format(A)
    lineB = "B) {}    # Output data image block                                 \n".format(B)
    lineC = "C) {}    # Sigma image name (made from data if blank or \"none\")  \n".format(C)
    lineD = "D) {}    # Input PSF image and (optional) diffusion kernel         \n".format(D)
    lineE = "E) {}    # PSF fine sampling factor relative to data               \n".format(E)
    lineF = "F) {}    # Bad pixel mask (FITS image or ASCII coord list)         \n".format(F)
    lineG = "G) {}    # File with parameter constraints (ASCII file)            \n".format(G)
    lineH = "H) {} {} {} {}   # Image region to fit (xmin xmax ymin ymax)               \n".format(xlo, xhi, ylo, yhi)
    lineI = "I) {} {}  # Size of the convolution box (x y)                       \n".format(convx, convy)
    lineJ = "J) {}     # Magnitude photometric zeropoint                         \n".format(J)
    lineK = "K) {} {}  # Plate scale (dx dy). \[arcsec per pixel\]               \n".format(platedx, platedy)
    lineO = "O) {}     # Display type (regular, curses, both)                    \n".format(O)
    lineP = "P) {}     # Choose 0=optimize, 1=model, 2=imgblock, 3=subcomps      \n".format(P)
    lineS = "S) {}     # Modify/create objects interactively?                    \n".format(S)
    lineY = " \n"

    line0 = "# INITIAL FITTING PARAMETERS                                                     \n"
    line1 = "# \n"
    line2 = "#   For object type, allowed functions are:                                      \n"
    line3 = "#       nuker, sersic, expdisk, devauc, king, psf, gaussian, moffat,             \n"
    line4 = "#       ferrer, powsersic, sky, and isophote.                                    \n"
    line5 = "# \n"
    line6 = "#  Hidden parameters will only appear when they're specified:                    \n"
    line7 = "#      C0 (diskyness/boxyness),                                                  \n"
    line8 = "#      Fn (n=integer, Azimuthal Fourier Modes),                                  \n"
    line9 = "#      R0-R10 (PA rotation, for creating spiral structures).                     \n"
    line10 = "# \n"

    line11 = "# column 1:  Parameter number                                                               \n"
    line12 = "# column 2:                                                                                 \n"
    line13 = "#          -- Parameter 0:    the allowed functions are: sersic, nuker, expdisk             \n"
    line14 = "#                             edgedisk, devauc, king, moffat, gaussian, ferrer, psf, sky    \n"
    line15 = "#          -- Parameter 1-10: value of the initial parameters                               \n"
    line16 = "#          -- Parameter C0:   For diskiness/boxiness                                        \n"
    line17 = "#                             <0 = disky                                                    \n"
    line18 = "#                             >0 = boxy                                                     \n"
    line19 = "#          -- Parameter Z:    Outputting image options, the options are:                    \n"
    line20 = "#                             0 = normal, i.e. subtract final model from the data to create \n"
    line21 = "#                             the residual image                                            \n"
    line22 = "#                             1 = Leave in the model -- do not subtract from the data       \n"
    line23 = "#                                                                                           \n"
    line24 = "# column 3: allow parameter to vary (yes = 1, no = 0)                                       \n"
    line25 = "# column 4: comment                                                                         \n"
    line26 = " \n"

    line27 = "==================================================================================================\n"

    hdl.write(lineZ)
    hdl.write(lineX)
    hdl.write(lineA)
    hdl.write(lineB)
    hdl.write(lineC)
    hdl.write(lineD)
    hdl.write(lineE)
    hdl.write(lineF)
    hdl.write(lineG)
    hdl.write(lineH)
    hdl.write(lineI)
    hdl.write(lineJ)
    hdl.write(lineK)
    hdl.write(lineO)
    hdl.write(lineP)
    hdl.write(lineS)
    hdl.write(lineY)

    hdl.write(line0)
    hdl.write(line1)
    hdl.write(line2)
    hdl.write(line3)
    hdl.write(line4)
    hdl.write(line5)
    hdl.write(line6)
    hdl.write(line7)
    hdl.write(line8)
    hdl.write(line9)
    hdl.write(line10)

    hdl.write(line11)
    hdl.write(line12)
    hdl.write(line13)
    hdl.write(line14)
    hdl.write(line15)
    hdl.write(line16)
    hdl.write(line17)
    hdl.write(line18)
    hdl.write(line19)
    hdl.write(line20)
    hdl.write(line21)
    hdl.write(line22)
    hdl.write(line23)
    hdl.write(line24)
    hdl.write(line25)
    hdl.write(line26)
    hdl.write(line27)

    return True


def PrintSky(hdl, ncomp, sky, Z, fit):
    "Print GALFIT sky function to filehandle"

    # k Check

    line00 = "# Object number: {}                                                             \n".format(ncomp)
    line01 = " 0)      sky            #    Object type                                        \n"
    line02 = " 1) {}   {}   # sky background        [ADU counts]                    \n".format(sky, fit)
    line03 = " 2) 0.000      0        # dsky/dx (sky gradient in x)                           \n"
    line04 = " 3) 0.000      0        # dsky/dy (sky gradient in y)                           \n"
    line05 = " Z) {}                  # Skip this model in output image?  (yes=1, no=0)       \n".format(Z)
    line06 = "\n"
    line07 = "================================================================================\n"

    hdl.write(line00)
    hdl.write(line01)
    hdl.write(line02)
    hdl.write(line03)
    hdl.write(line04)
    hdl.write(line05)
    hdl.write(line06)
    hdl.write(line07)

    return True


def PrintSersic(hdl, ncomp, xpos, ypos, magser, reser, nser, axratser, angleser, Z, fit, serfit):
    "print GALFIT Sersic function to filehandle"
    # k Check

    # print to filehandle
    # a sersic function given
    # by the parameters

    line00 = "# Object number: {}                                                             \n".format(
            ncomp)
    line01 = " 0)     sersic      #  Object type                                     \n"
    line02 = " 1) {:.2f}  {:.2f}  {}  {}  #  position x, y     [pixel]                       \n".format(
        xpos, ypos, fit, fit)
    line03 = " 3) {:.2f}  {}    #  total magnitude                                 \n".format(
        magser, fit)
    line04 = " 4) {:.2f}  {}    #  R_e         [Pixels]                            \n".format(
            reser, fit)
    line05 = " 5) {}     {}   #  Sersic exponent (deVauc=4, expdisk=1)           \n".format(
        nser, serfit)
    #line05 = " 5) {}       {}              #  Sersic exponent (deVauc=4, expdisk=1)           \n".format(
    #    nser, fit)
    line06 = " 6)  0.0000       0           #  ----------------                                \n"
    line07 = " 7)  0.0000       0           #  ----------------                                \n"
    line08 = " 8)  0.0000       0           #  ----------------                                \n"
    line09 = " 9) {:.2f}   {}   #  axis ratio (b/a)                                \n".format(
        axratser, fit)
    line10 = "10) {:.2f}    {}  #  position angle (PA)  [Degrees: Up=0, Left=90]   \n".format(
        angleser, fit)
    lineZ = " Z) {}             #  Skip this model in output image?  (yes=1, no=0) \n".format(
        Z)
    line11 = "\n"

    hdl.write(line00)
    hdl.write(line01)
    hdl.write(line02)
    hdl.write(line03)
    hdl.write(line04)
    hdl.write(line05)
    hdl.write(line06)
    hdl.write(line07)
    hdl.write(line08)
    hdl.write(line09)
    hdl.write(line10)
    hdl.write(lineZ)
    hdl.write(line11)

    return True

def PrintGauss(hdl, ncomp, xpos, ypos, magass, fwhm, axratgass, anglegass, Z, fit):
    "print GALFIT GAUSS function to filehandle"

    # print to filehandle
    # a sersic function given
    # by the parameters

    line00 = "# Object number: {}                                                             \n".format(
            ncomp)
    line01 = " 0)     gaussian   #  Object type                                     \n"
    line02 = " 1) {:.2f}  {:.2f}  {}  {}  #  position x, y     [pixel]                       \n".format(
        xpos, ypos, fit, fit)
    line03 = " 3) {:.2f}       {}       #  total magnitude                                 \n".format(
        magass, fit)
    line04 = " 4) {:.2f}       {}       #  FWHM         [Pixels]                            \n".format(
            fwhm, fit)
    line05 = " 9) {:.2f}       {}       #  axis ratio (b/a)                                \n".format(
        axratgass, fit)
    line06 = "10) {:.2f}       {}       #  position angle (PA)  [Degrees: Up=0, Left=90]   \n".format(
        anglegass, fit)
    line07 = " Z) {}               #  Skip this model in output image?  (yes=1, no=0) \n".format(
        Z)
    line08 = "\n"

    hdl.write(line00)
    hdl.write(line01)
    hdl.write(line02)
    hdl.write(line03)
    hdl.write(line04)
    hdl.write(line05)
    hdl.write(line06)
    hdl.write(line07)
    hdl.write(line08)

    return True

def PrintExp(hdl, ncomp, xpos, ypos, magexp, rsexp, axratexp, angleexp, Z, fit):
    "print GALFIT exponential function to filehandle"

    # k Check

    # print to filehandle
    # a exponential function given
    # by the parameters

    line00 = "# Object number: $ncomp                                                        \n"
    line01 = " 0)     expdisk        # Object type                                     \n"
    line02 = " 1) {:.2f}  {:.2f}  {}  {}  # position x, y     [pixel]                       \n".format(
        xpos, ypos, fit, fit)
    line03 = " 3) {:.2f}        {}   # total magnitude                                 \n".format(
        magexp, fit)
    line04 = " 4) {:.2f}        {}   #      Rs  [Pixels]                               \n".format(
        rsexp, fit)
    line05 = " 9) {:.2f}        {}   # axis ratio (b/a)                                \n".format(
        axratexp, fit)
    line06 = "10) {:.2f}        {}   # position angle (PA)  [Degrees: Up=0, Left=90]   \n".format(
        angleexp, fit)
    line07 = " Z) {}                 # Skip this model in output image?  (yes=1, no=0) \n".format(
        Z)
    line08 = "\n"

    hdl.write(line00)
    hdl.write(line01)
    hdl.write(line02)
    hdl.write(line03)
    hdl.write(line04)
    hdl.write(line05)
    hdl.write(line06)
    hdl.write(line07)
    hdl.write(line08)

    return True



def GetFits(Image, Imageout, xlo, xhi, ylo, yhi):
    "Get a piece from the image"
# k Check


    if os.path.isfile(Imageout):
        print("{} deleted; a new one is created \n".format(Imageout))
        runcmd = "rm {}".format(Imageout)
        errrm = sp.run([runcmd], shell=True, stdout=sp.PIPE,
                        stderr=sp.PIPE, universal_newlines=True)


    hdu = fits.open(Image)
    dat = hdu[0].data[ylo - 1:yhi, xlo - 1:xhi]
    hdu[0].data = dat
    hdu.writeto(Imageout, overwrite=True)
    hdu.close()

def GetExpTime(Image):
    # k Check
    "Get exposition time from the image"

    try:
        hdu = fits.open(Image)
        exptime = hdu[0].header["EXPTIME"]
        hdu.close()
    except: 
        exptime = 1
    return float(exptime)

def makeConstraints(consfile: str, numcomp: int) -> True:
    "creates a contraints file with the number of components"


    fout = open(consfile, "w")


    line00 = "#Component/    parameter   constraint	Comment           \n"
    line01 = "# operation	(see below)   range         \n"

    linempty = "                  \n"				

    comp = np.arange(2,numcomp+1)

    cad = "1"
    for idx, item in enumerate(comp):
        cad = cad + "_" + str(item)

    line02 = "    " + cad + "    " + "x" + "    "  + "offset \n"				
    line03 = "    " + cad + "    " + "y" + "    "  + "offset \n"				


    fout.write(line00)
    fout.write(line01)
    fout.write(linempty)
    fout.write(line02)
    fout.write(line03)

    fout.close()



    return True





def CheckFlag(val, check):
    "Check for flag contained in val, returns True if found "

    flag = False
    mod = 1
    maxx=128


    while (mod != 0):

        res = int(val / maxx)

        if (maxx == check and res == 1):

            flag = True

        mod = val % maxx

        val = mod
        maxx = maxx / 2

    return flag


def GetInfoEllip(regfile):

    if not os.path.exists(regfile):
        print ('%s: reg filename does not exist!' %(regfile))
        sys.exit()

    f1 = open(regfile,'r')

    lines = f1.readlines()

    f1.close()

    flag = False
    found = False

    #reading reg file
    for line in lines:


        line = line.split("#")
        line = line[0]

        b1 = line.split("(")
        p = line.split(",")

        x1 = p[0]
        if b1[0] == "ellipse":

            x0 = "ellipse"
            x2 = x1[8:]
            flag = True
            found = True

        if (flag == True):
            x3 = p[4]
            x4 = x3[:-2]


            v0 = x0

            v1 = float(x2)
            v2 = float(p[1])
            v3 = float(p[2])
            v4 = float(p[3])
            v5 = float(x4)

            flag = False

    if found:
        obj  = v0
        xpos = v1
        ypos = v2
        rx = v3
        ry = v4
        angle = v5

        if rx >= ry:
            axratio = ry/rx 
            eps = 1 - axratio 
            theta = angle + 90 
        else:
            axratio = rx/ry 
            eps = 1 - axratio 
            theta = angle 

        #avoids ds9 regions with Area = 0
        if rx<1:
            rx=1
        if ry<1:
            ry=1


        return obj, xpos, ypos, rx, ry, angle
        #return eps, theta, xpos, ypos
    else:
        print("ellipse region was not found in file. Exiting.. ")
        sys.exit()

    return 0, 0, 0, 0



##new: 

#Image = MakeEllip(Image,Value,xpos[idx],ypos[idx],rx[idx],ry[idx],angle[idx],ncol,nrow)

#def MakeEllip(Image,Value,xpos,ypos,rx,ry,angle,ncol,nrow):
#    "Make an ellipse in an image"

#    xx, yy, Rkron, theta, e = Ds9ell2Kronell(xpos,ypos,rx,ry,angle)
#    (xmin, xmax, ymin, ymax) = GetSize(xx, yy, Rkron, theta, e, ncol, nrow)
#    Image = MakeKron(Image, Value, xx, yy, Rkron, theta, e, xmin, xmax, ymin, ymax)

#    return Image


def Ds9ell2Kronellv2(xpos,ypos,rx,ry,angle):


    if rx >= ry:

        q = ry/rx
        e = 1 - q
        Rkron = rx
        theta = angle - 90
        xx = xpos
        yy = ypos
    else:
        q = rx/ry
        e = 1 - q
        Rkron = ry
        theta = angle# + 90
        xx = xpos
        yy = ypos
 
     
    return xx, yy, Rkron, theta, e 


def GetSize(x, y, R, theta, ell, ncol, nrow):
    "this subroutine get the maximun"
    "and minimim pixels for Kron and sky ellipse"
    # k Check
    q = (1 - ell)
    bim = q * R

    theta = theta * (np.pi / 180)  # rads!!

# getting size

    xmin = x - np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    xmax = x + np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    ymin = y - np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)

    ymax = y + np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)

    mask = xmin < 1
    if mask.any():
        xmin = 1

    mask = xmax > ncol
    if mask.any():
        xmax = ncol

    mask = ymin < 1
    if mask.any():
        ymin = 1

    mask = ymax > nrow
    if mask.any():
        ymax = nrow

    return (xmin, xmax, ymin, ymax)



def GetPmax(image, mask, xmin, xmax, ymin, ymax):


    xmin = int(xmin)
    xmax = int(xmax)
    ymin = int(ymin)
    ymax = int(ymax)

    chuckimg = image[ymin - 1:ymax, xmin - 1:xmax]
    if mask.any():
        chuckmsk = mask[ymin - 1:ymax, xmin - 1:xmax]

        invmask = np.logical_not(chuckmsk)

        invmask = invmask*1

        chuckimg = chuckimg*invmask

    maxy, maxx = np.where(chuckimg == np.max(chuckimg))
    
    xpos = maxx[0] + xmin - 1
    ypos = maxy[0] + ymin - 1

    return (xpos, ypos)





#############################################################################
######################### End of program  ###################################
#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/
##############################################################################
if __name__ == '__main__':
    main()



#! /usr/bin/env python

import numpy as np 

from  galfitools.mge.mge2galfit import Ds9ell2Kronellv2
from  galfitools.mge.mge2galfit import GetInfoEllip
from  galfitools.mge.mge2galfit import GetSize
from  galfitools.mge.mge2galfit import GetPmax
from  galfitools.mge.mge2galfit import GetAxis

from astropy.io import fits
import os

def SkyRing(image, mask, ds9regfile, width, center, outliers):
    """Computes the sky using gradient over rings """


    assert os.path.exists(image),"image file does not exists"

    if mask:
        assert os.path.exists(mask),"mask file does not exists"

    ##end input
    obj, xpos, ypos, rx, ry, angle = GetInfoEllip(ds9regfile)
    xx, yy, Rkron, theta, eps = Ds9ell2Kronellv2(xpos,ypos,rx,ry,angle)
    q = 1 - eps

    hdu = fits.open(image)
    datimg = hdu[0].data
    hdu.close()

    if mask:
        hdumask = fits.open(mask)
        maskimg = hdumask[0].data
        hdumask.close()
    else:
        maskimg = np.array([])


    (ncol, nrow) = GetAxis(image)

    if center:
        print('center of ds9 ellipse region will be used')
        xpeak, ypeak = xpos, ypos
    else:        
        (xmin, xmax, ymin, ymax) = GetSize(xx, yy, Rkron, theta, eps, ncol, nrow)
        xpeak, ypeak = GetPmax(datimg, maskimg, xmin, xmax, ymin, ymax)
 



    mean, std, median, rad = SkyCal().GetEllipSky(datimg,maskimg,xpeak,ypeak,
                                                        theta,q,Rkron,width,
                                                        "skyring.fits","ringmask.fits", outliers=outliers)



    return mean, std, median, rad



class SkyCal:
    "computes sky background "

    def GetEllipSky(self, ImageFile, MaskImg, xx, yy, thetadeg, q, Rinit, width,namering,ringmask,outliers=True):
        "Gradient sky method"

        self.xx = xx 
        self.yy = yy

        self.thetadeg =  thetadeg + 90 
        self.q = q
        self.e = (1 - self.q)
        self.Rinit = Rinit
        self.width = width 

        self.NumRings = 5  # number of rings per loop # read this from function?

        self.ringmask = ringmask 

        self.outliers = outliers

        ###

        #hdumask = fits.open(MaskFile)
        #self.maskimg = hdumask[0].data
        #hdumask.close()

        if MaskImg.any():
            self.maskimg = MaskImg.copy()
        else:
            self.maskimg = np.array([])



        #hdu = fits.open(ImageFile)
        #self.img = hdu[0].data
        #hdu.close()
        self.img = ImageFile.copy()

        ####
       
        (self.nrow,self.ncol) = self.img.shape


        xmin,xmax,ymin,ymax,Rkron = self.GetXYRBorder()

        self.R = Rkron

        if self.Rinit > Rkron: # avoid radius greater than the border

            self.Rinit= Rkron/3 #is this number ok?
            print("Rinit is greater than image size")
            print("using Rinit = {:0.2f} ".format(self.Rinit))
            print("change this value with --radinit ")





        (xmin, xmax, ymin, ymax) = self.GetSize(self.xx, self.yy, Rkron, self.thetadeg, self.q, self.ncol, self.nrow) # obtain corners of R

        theta = self.thetadeg * np.pi / 180  # Rads!!!


        if self.maskimg.any(): 
            patch = self.maskimg[ymin - 1:ymax, xmin - 1:xmax] # logical patch mask image

            self.invpatch = np.logical_not(patch)
        else:
            self.invpatch = np.array([])

        Rings=np.arange(self.Rinit,self.R,self.width) # anillos de tamaño width

        sky = np.array([])
        skymed = np.array([])
        skystd = np.array([])
        radius = np.array([])


        count = 0
        idx=0


        ########################################
        #creating ring masks file

        masksky = np.zeros(np.shape(self.img))

        val = 1 
        for ridx, ritem in enumerate(Rings):


            bring = (Rings[ridx] + self.width) * self.q
            aring = Rings[ridx] + self.width 

            #incresing number of points per rad 
            points = 2*np.pi * np.sqrt(0.5*(aring**2 + bring**2)) #Aprox. 
            points = 2*points #doubling the number of points
            points = int(round(points))
           
            alpha = np.linspace(0,2*np.pi,points)
            for tidx, item in enumerate(range(self.width)):

                bim=(Rings[ridx]+item)*self.q

                tempxell = self.xx + (Rings[ridx]+item) * np.cos(alpha) * np.cos(theta) - bim * \
                  np.sin(alpha) * np.sin(theta)

                tempyell = self.yy + (Rings[ridx]+item) * np.cos(alpha) * np.sin(theta) + bim * \
                  np.sin(alpha) * np.cos(theta)

                tempxell = tempxell.round().astype("int")
                tempyell = tempyell.round().astype("int")

                tempxell,tempyell = self.CorSize(tempxell,tempyell)
                masksky[tempyell - 1, tempxell - 1] = val
                
            val += 1 


        ########################################
        ########################################
        if self.outliers:   # eliminate top 80% and bottom 20%
             print("removing top 80% and bottom 20% for every ring")
 


        #computing sky in every ring
        for ind, item in enumerate(Rings):


            maskring,idx=self.GetRingMask(masksky[ymin - 1:ymax, xmin - 1:xmax],idx)


            flatimg=self.img[ymin - 1:ymax, xmin - 1:xmax][maskring].flatten()  
            flatimg.sort()

            tot=len(flatimg)

            top=round(.8*tot)
            bot=round(.2*tot)


            if self.outliers:   # eliminate top 80% and bottom 20%
                imgpatch = flatimg[bot:top]
            else:
                imgpatch = flatimg

            mean=np.mean(imgpatch)
            std=np.std(imgpatch)

            median=np.median(imgpatch)

            sky=np.append(sky,mean)
            skymed=np.append(skymed,median)
            skystd=np.append(skystd,std)
            radius=np.append(radius,Rings[idx] + self.width/2)

            print("Ring = {}; rad = {:.2f}; sky mean = {:.2f}; sky std = {:.2f}; median: {:.2f} ".format(idx+1,Rings[idx] + self.width/2,mean,std, median))


            # calcular gradiente
            if (count >= self.NumRings):

                # [1:-1] avoiding the first and last element for gradient 
                gradmask = np.gradient(sky[1:-1]) >= 0 
               
                count = 0
                tempidx=np.where(np.gradient(sky[1:-1]) >= 0) 
                
                if (sky[1:-1][gradmask].any()): 
                    
                    savidx=tempidx[0][0]
                    maskring,none =self.GetRingMask(masksky[ymin - 1:ymax, xmin - 1:xmax],savidx)


                    print("sky computed in ring {} ".format(savidx+2))
                    
                    print("Ring radius = {:.2f} ".format(radius[1:-1][savidx]))
                    print("the counts value within ring represent the long axis") 
                    self.img[ymin - 1:ymax, xmin - 1:xmax][maskring] = radius[1:-1][savidx] 
                    break

            count += 1
            idx +=1

            if idx == (len(Rings)-1): 
                print("The edge of image has been reached. Sky can not be computed")
                return 0,0,0,0

        hduout = fits.PrimaryHDU()
        hduout.data=self.img
        hduout.writeto(namering,overwrite=True) 


        #hdu.writeto(namering,overwrite=True) 

        finmean,finmedian,finstd,finRad = sky[1:-1][gradmask],skymed[1:-1][gradmask],skystd[1:-1][gradmask],radius[1:-1][gradmask]


        return finmean[0],finstd[0],finmedian[0],finRad[0]


    def GetRingMask(self,masksky,idx):
        ''' obtains the ring selected by index idx'''

        ring= idx + 1

        maskring = masksky == ring 

        if self.invpatch.any():
            maskring = maskring*self.invpatch

        ringcont=0

        while( not(maskring.any()) and (ringcont < 10)):

            if (ringcont == 0):
                    print("Selecting next ring ")

            idx += 1
            ring= idx + 1

            maskring = masksky == ring 

            if self.invpatch.any():
                maskring=maskring*self.invpatch


            ringcont+=1 # avoid eternal loop

        if (ringcont == 10):
            print("max. iteration reached. I couldn't find a ring") 
            return 0,0 # It couldn't found any ring ending 
 




        return maskring, idx


    def CorSize(self,xell,yell):
        '''Correct size for image borders'''
        masksx = xell < 1
        masksy = yell < 1

        xell[masksx] = 1
        yell[masksy] = 1

        masksx = xell > self.ncol
        masksy = yell > self.nrow 

        xell[masksx] = self.ncol 
        yell[masksy] = self.nrow

        return xell,yell




    def GetXYRBorder(self):
        "this subroutine get the coordinates of the border"

        q =  self.q

        if self.thetadeg > 360:
            self.thetadeg = self.thetadeg - 360 #quick fix

        theta = self.thetadeg * (np.pi / 180)  # rads!!

        thetax=np.sqrt((np.cos(theta))**2 + (q**2)*(np.sin(theta))**2 )
        thetay=np.sqrt((q**2)*(np.cos(theta))**2 + (np.sin(theta))**2 )


        if self.thetadeg < 0:
            self.thetadeg = 360 - self.thetadeg


        if (self.thetadeg > 315 or self.thetadeg <= 45):

            xmax=self.ncol
            xmin =1
            R1 = (xmax - self.xx)/thetax
            R2 = (self.xx - xmin)/thetax

            if (np.abs(R1)>=np.abs(R2)):
                R = R1
            else:
                R = R2

        elif (self.thetadeg > 45 and self.thetadeg <= 135):
            ymax=self.nrow
            ymin =1
            R1 = (ymax - self.yy)/thetay
            R2 = (self.yy - ymin)/thetay

            if (np.abs(R1)>=np.abs(R2)):
                R = R1
            else:
                R = R2

        elif (self.thetadeg > 135 and self.thetadeg <= 225):
            xmax=1
            xmin =self.ncol

            R1 = (xmax - self.xx)/thetax
            R2 = (self.xx - xmin)/thetax

            if (np.abs(R1)>=np.abs(R2)):
                R = R1
            else:
                R = R2

     
        elif (self.thetadeg > 225 and self.thetadeg <= 315):
            ymax=1
            ymin =self.nrow
            R1 = (ymax - self.yy)/thetay
            R2 = (self.yy - ymin)/thetay

            if (np.abs(R1)>=np.abs(R2)):
               R = R1
            else:
               R = R2

        R = np.abs(R) #avoids negative numbers
        bim = q * R

        # getting size

        xmin = self.xx - np.sqrt((R**2) * (np.cos(theta))**2 +
                         (bim**2) * (np.sin(theta))**2)

        xmax = self.xx + np.sqrt((R**2) * (np.cos(theta))**2 +
                         (bim**2) * (np.sin(theta))**2)

        ymin = self.yy - np.sqrt((R**2) * (np.sin(theta))**2 +
                         (bim**2) * (np.cos(theta))**2)

        ymax = self.yy + np.sqrt((R**2) * (np.sin(theta))**2 +
                         (bim**2) * (np.cos(theta))**2)

        mask = xmin < 1
        if mask.any():
            if isinstance(xmin,np.ndarray):
                xmin[mask] = 1
            else:
                xmin = 1

        mask = xmax > self.ncol
        if mask.any():
            if isinstance(xmax,np.ndarray):
                xmax[mask] = self.ncol
            else:
                xmax = self.ncol

        mask = ymin < 1
        if mask.any():
            if isinstance(ymin,np.ndarray):
                ymin[mask] = 1
            else:
                ymin = 1

        mask = ymax > self.nrow
        if mask.any():
            if isinstance(ymax,np.ndarray):
                ymax[mask] = self.nrow
            else:
                ymax =self.nrow

        xmin=np.int32(np.round(xmin))
        ymin=np.int32(np.round(ymin))
        xmax=np.int32(np.round(xmax))
        ymax=np.int32(np.round(ymax))


        return (xmin,xmax,ymin,ymax,np.int32(R))


    def GetSize(self,x, y, R, theta, q, ncol, nrow):
        '''this subroutine get the maximun
        and minimim pixels for Kron and sky ellipse'''

        bim = q * R

        theta = theta * (np.pi / 180)  # rads!!

        # getting size
        xmin = x - np.sqrt((R**2) * (np.cos(theta))**2 +
                           (bim**2) * (np.sin(theta))**2)

        xmax = x + np.sqrt((R**2) * (np.cos(theta))**2 +
                           (bim**2) * (np.sin(theta))**2)

        ymin = y - np.sqrt((R**2) * (np.sin(theta))**2 +
                           (bim**2) * (np.cos(theta))**2)

        ymax = y + np.sqrt((R**2) * (np.sin(theta))**2 +
                           (bim**2) * (np.cos(theta))**2)

        mask = xmin < 1
        if mask.any():
            if isinstance(xmin,np.ndarray):
                xmin[mask] = 1
            else:
                xmin = 1

        mask = xmax > ncol

        if mask.any():
            if isinstance(xmax,np.ndarray):
                xmax[mask] = ncol
            else:
                xmax = ncol

        mask = ymin < 1
        if mask.any():
            if isinstance(ymin,np.ndarray):
                ymin[mask] = 1
            else:
                ymin = 1

        mask = ymax > nrow
        if mask.any():
            if isinstance(ymax,np.ndarray):
                ymax[mask] = nrow
            else:
                ymax = nrow

        xmin=np.int32(np.round(xmin))
        ymin=np.int32(np.round(ymin))
        xmax=np.int32(np.round(xmax))
        ymax=np.int32(np.round(ymax))


        return (xmin, xmax, ymin, ymax)


    ##### End of sky class ##################
    #########################################
    #########################################

